# -*- encoding: utf-8 -*-
import collections
from abjad.tools import indicatortools
from abjad.tools import instrumenttools
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools import scoretools
from abjad.tools.abctools.AbjadObject import AbjadObject
from abjad.tools.topleveltools import attach


class StringOrchestraScoreTemplate(AbjadObject):
    '''String orchestra score template.

    ::

        >>> template = templatetools.StringOrchestraScoreTemplate(
        ...     violin_count=6,
        ...     viola_count=4,
        ...     cello_count=3,
        ...     contrabass_count=2,
        ...     )
        >>> score = template()

    ::

        >>> score
        Score-"String Orchestra Score"<<4>>

    ..  doctest::
                
        >>> print format(score)        
        \context Score = "String Orchestra Score" <<
            \context StaffGroup = "Violin Staff Group" <<
                \context Staff = "Violin 1 Voice" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin 1 }
                    \set Staff.shortInstrumentName = \markup { Vln. 1 }
                    \context Voice = "Violin 1 Voice" {
                    }
                }
                \context Staff = "Violin 2 Voice" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin 2 }
                    \set Staff.shortInstrumentName = \markup { Vln. 2 }
                    \context Voice = "Violin 2 Voice" {
                    }
                }
                \context Staff = "Violin 3 Voice" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin 3 }
                    \set Staff.shortInstrumentName = \markup { Vln. 3 }
                    \context Voice = "Violin 3 Voice" {
                    }
                }
                \context Staff = "Violin 4 Voice" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin 4 }
                    \set Staff.shortInstrumentName = \markup { Vln. 4 }
                    \context Voice = "Violin 4 Voice" {
                    }
                }
                \context Staff = "Violin 5 Voice" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin 5 }
                    \set Staff.shortInstrumentName = \markup { Vln. 5 }
                    \context Voice = "Violin 5 Voice" {
                    }
                }
                \context Staff = "Violin 6 Voice" {
                    \clef "treble"
                    \set Staff.instrumentName = \markup { Violin 6 }
                    \set Staff.shortInstrumentName = \markup { Vln. 6 }
                    \context Voice = "Violin 6 Voice" {
                    }
                }
            >>
            \context StaffGroup = "Viola Staff Group" <<
                \context Staff = "Viola 1 Voice" {
                    \clef "alto"
                    \set Staff.instrumentName = \markup { Viola 1 }
                    \set Staff.shortInstrumentName = \markup { Vla. 1 }
                    \context Voice = "Viola 1 Voice" {
                    }
                }
                \context Staff = "Viola 2 Voice" {
                    \clef "alto"
                    \set Staff.instrumentName = \markup { Viola 2 }
                    \set Staff.shortInstrumentName = \markup { Vla. 2 }
                    \context Voice = "Viola 2 Voice" {
                    }
                }
                \context Staff = "Viola 3 Voice" {
                    \clef "alto"
                    \set Staff.instrumentName = \markup { Viola 3 }
                    \set Staff.shortInstrumentName = \markup { Vla. 3 }
                    \context Voice = "Viola 3 Voice" {
                    }
                }
                \context Staff = "Viola 4 Voice" {
                    \clef "alto"
                    \set Staff.instrumentName = \markup { Viola 4 }
                    \set Staff.shortInstrumentName = \markup { Vla. 4 }
                    \context Voice = "Viola 4 Voice" {
                    }
                }
            >>
            \context StaffGroup = "Cello Staff Group" <<
                \context Staff = "Cello 1 Voice" {
                    \clef "bass"
                    \set Staff.instrumentName = \markup { Cello 1 }
                    \set Staff.shortInstrumentName = \markup { Vc. 1 }
                    \context Voice = "Cello 1 Voice" {
                    }
                }
                \context Staff = "Cello 2 Voice" {
                    \clef "bass"
                    \set Staff.instrumentName = \markup { Cello 2 }
                    \set Staff.shortInstrumentName = \markup { Vc. 2 }
                    \context Voice = "Cello 2 Voice" {
                    }
                }
                \context Staff = "Cello 3 Voice" {
                    \clef "bass"
                    \set Staff.instrumentName = \markup { Cello 3 }
                    \set Staff.shortInstrumentName = \markup { Vc. 3 }
                    \context Voice = "Cello 3 Voice" {
                    }
                }
            >>
            \context StaffGroup = "Contrabass Staff Group" <<
                \context Staff = "Contrabass 1 Voice" {
                    \clef "bass_8"
                    \set Staff.instrumentName = \markup { Contrabass 1 }
                    \set Staff.shortInstrumentName = \markup { Cb. 1 }
                    \context Voice = "Contrabass 1 Voice" {
                    }
                }
                \context Staff = "Contrabass 2 Voice" {
                    \clef "bass_8"
                    \set Staff.instrumentName = \markup { Contrabass 2 }
                    \set Staff.shortInstrumentName = \markup { Cb. 2 }
                    \context Voice = "Contrabass 2 Voice" {
                    }
                }
            >>
        >>

    Returns score template.
    '''

    ### INITIALIZER ###

    def __init__(
        self,
        violin_count=6,
        viola_count=4,
        cello_count=3,
        contrabass_count=2,
        ):
        assert 0 <= violin_count
        assert 0 <= viola_count
        assert 0 <= cello_count
        assert 0 <= contrabass_count
        self._violin_count = int(violin_count)
        self._viola_count = int(viola_count)
        self._cello_count = int(cello_count)
        self._contrabass_count = int(contrabass_count)

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Calls string orchestra template.

        Returns score.
        '''

        string_orchestra_score = scoretools.Score(
            name='String Orchestra Score',
            )

        if self.violin_count:
            violin_staff_group = scoretools.StaffGroup(
                name='Violin Staff Group',
                )
            for i in range(1, self.violin_count + 1):
                violin_voice = scoretools.Voice(
                    name='Violin {} Voice'.format(i),
                    )
                violin_staff = scoretools.Staff(
                    [violin_voice], name='Violin {} Voice'.format(i))
                clef = indicatortools.Clef('treble')
                attach(clef, violin_staff)
                violin = instrumenttools.Violin(
                    instrument_name_markup='Violin {}'.format(i),
                    short_instrument_name_markup='Vln. {}'.format(i),
                    )
                attach(violin, violin_staff)
                violin_staff_group.append(violin_staff)
            string_orchestra_score.append(violin_staff_group)

        if self.viola_count:
            viola_staff_group = scoretools.StaffGroup(
                name='Viola Staff Group',
                )
            for i in range(1, self.viola_count + 1):
                viola_voice = scoretools.Voice(
                    name='Viola {} Voice'.format(i),
                    )
                viola_staff = scoretools.Staff(
                    [viola_voice], name='Viola {} Voice'.format(i))
                clef = indicatortools.Clef('alto')
                attach(clef, viola_staff)
                viola = instrumenttools.Viola(
                    instrument_name_markup='Viola {}'.format(i),
                    short_instrument_name_markup='Vla. {}'.format(i),
                    )
                attach(viola, viola_staff)
                viola_staff_group.append(viola_staff)
            string_orchestra_score.append(viola_staff_group)

        if self.cello_count:
            cello_staff_group = scoretools.StaffGroup(
                name='Cello Staff Group',
                )
            for i in range(1, self.cello_count + 1):
                cello_voice = scoretools.Voice(
                    name='Cello {} Voice'.format(i),
                    )
                cello_staff = scoretools.Staff(
                    [cello_voice], name='Cello {} Voice'.format(i))
                clef = indicatortools.Clef('bass')
                attach(clef, cello_staff)
                cello = instrumenttools.Cello(
                    instrument_name_markup='Cello {}'.format(i),
                    short_instrument_name_markup='Vc. {}'.format(i),
                    )
                attach(cello, cello_staff)
                cello_staff_group.append(cello_staff)
            string_orchestra_score.append(cello_staff_group)

        if self.contrabass_count:
            contrabass_staff_group = scoretools.StaffGroup(
                name='Contrabass Staff Group',
                )
            for i in range(1, self.contrabass_count + 1):
                contrabass_voice = scoretools.Voice(
                    name='Contrabass {} Voice'.format(i),
                    )
                contrabass_staff = scoretools.Staff(
                    [contrabass_voice], name='Contrabass {} Voice'.format(i))
                clef = indicatortools.Clef('bass_8')
                attach(clef, contrabass_staff)
                contrabass = instrumenttools.Contrabass(
                    instrument_name_markup='Contrabass {}'.format(i),
                    short_instrument_name_markup='Cb. {}'.format(i),
                    )
                attach(contrabass, contrabass_staff)
                contrabass_staff_group.append(contrabass_staff)
            string_orchestra_score.append(contrabass_staff_group)

        # return string quartet score
        return string_orchestra_score

    ### PUBLIC PROPERTIES ###

    @property
    def cello_count(self):
        r'''Number of cellos in string orchestra.

        Returns nonnegative integer.
        '''
        return self._cello_count

    @property
    def contrabass_count(self):
        r'''Number of contrabasses in string orchestra.

        Returns nonnegative integer.
        '''
        return self._contrabass_count

    @property
    def viola_count(self):
        r'''Number of violas in string orcestra.

        Returns nonnegative integer.
        '''
        return self._viola_count

    @property
    def violin_count(self):
        r'''Number of violins in string orchestra.

        Returns nonnegative integer.
        '''
        return self._violin_count



# -*- encoding: utf-8 -*-
from abjad import *


def test_agenttools_MutationAgent_respell_with_flats_01():

    note = Note(('cs', 4), 4)
    mutate(note).respell_with_flats()

    assert note.written_pitch == NamedPitch('df', 4)


def test_agenttools_MutationAgent_respell_with_flats_02():

    chord = Chord([('cs', 4), ('f', 4), ('as', 4)], (1, 4))
    mutate(chord).respell_with_flats()

    assert chord.written_pitches == (
        NamedPitch('df', 4),
        NamedPitch('f', 4),
        NamedPitch('bf', 4),
        )


def test_agenttools_MutationAgent_respell_with_flats_03():

    staff = Staff("c'8 cs'8 d'8 ef'8 e'8 f'8 fs'8 g'8 af'8 a'8 bf'8 b'")
    mutate(staff).respell_with_flats()

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            c'8
            df'8
            d'8
            ef'8
            e'8
            f'8
            gf'8
            g'8
            af'8
            a'8
            bf'8
            b'8
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_PaperBlock_01():

    paper_block = lilypondfiletools.PaperBlock()
    paper_block.top_margin = 15
    paper_block.left_margin = 15
    paper_block.markup_system_spacing__basic_distance = 8

    assert systemtools.TestManager.compare(
        paper_block,
        r'''
        \paper {
            left-margin = #15
            markup-system-spacing #'basic-distance = #8
            top-margin = #15
        }
        '''
        )


def test_lilypondfiletools_PaperBlock_02():
    r'''Make sure scheme vectors format correctly.
    '''

    paper_block = lilypondfiletools.PaperBlock()
    paper_block.system_system_spacing = layouttools.make_spacing_vector(0, 0, 12, 0)

    assert systemtools.TestManager.compare(
        paper_block,
        r'''
        \paper {
            system-system-spacing = #'((basic-distance . 0) (minimum-distance . 0) (padding . 12) (stretchability . 0))
        }
        '''
        )


def test_lilypondfiletools_PaperBlock_03():

    paper_block = lilypondfiletools.PaperBlock()
    paper_block.append(schemetools.Scheme([
        'define', 'fonts', [
            'make-pango-font-tree',
            schemetools.Scheme('Baskerville', force_quotes=True),
            schemetools.Scheme('Baskerville', force_quotes=True),
            schemetools.Scheme('Baskerville', force_quotes=True),
            schemetools.Scheme(['/', 14, 20])
            ]
        ]))

    assert systemtools.TestManager.compare(
        paper_block,
        r'''
        \paper {
            #(define fonts (make-pango-font-tree "Baskerville" "Baskerville" "Baskerville" (/ 14 20)))
        }
        '''
        )



# -*- encoding: utf-8 -*-
import fractions
import functools
import math
import numbers
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import schemetools
from abjad.tools.abctools.AbjadObject import AbjadObject


@functools.total_ordering
class Tempo(AbjadObject):
    r'''A tempo indication.

    ::

        >>> score = Score([])
        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> score.append(staff)
        >>> tempo = Tempo(Duration(1, 8), 52)
        >>> attach(tempo, staff[0])
        >>> show(score) # doctest: +SKIP

    ..  doctest::

        >>> print format(score)
        \new Score <<
            \new Staff {
                \tempo 8=52
                c'8
                d'8
                e'8
                f'8
            }
        >>

    Tempo indications are scoped to the **score context** by default.
    '''

    ### CLASS VARIABLES ###

    _default_positional_input_arguments = (
        (1, 8),
        68,
        )

    _format_slot = 'opening'

    ### INITIALIZER ###

    def __init__(self, *args, **kwargs):
        from abjad.tools import scoretools
        self._default_scope = scoretools.Score
        if len(args) == 1 and isinstance(args[0], type(self)):
            tempo_indication = args[0]
            duration = durationtools.Duration(tempo_indication.duration)
            textual_indication = tempo_indication.textual_indication
            units_per_minute = tempo_indication.units_per_minute
        elif len(args) == 1 and isinstance(args[0], str):
            duration = None
            textual_indication = args[0]
            units_per_minute = None
            assert isinstance(textual_indication, (str, type(None)))
        elif len(args) == 1 and isinstance(args[0], tuple) and \
            len(args[0]) == 2:
            textual_indication = None
            duration, units_per_minute = args[0]
        elif len(args) == 1 and isinstance(args[0], tuple) and \
            len(args[0]) == 3:
            textual_indication, duration, units_per_minute = args[0]
        elif len(args) in [2, 3]:
            if len(args) == 3:
                textual_indication, duration, units_per_minute = args
            else:
                textual_indication = None
                duration, units_per_minute = args
        else:
            message = 'can not initialize tempo indication.'
            raise ValueError(message)
        assert isinstance(textual_indication, (str, type(None)))
        if duration:
            try:
                duration = durationtools.Duration(duration)
            except TypeError:
                duration = durationtools.Duration(*duration)
        assert isinstance(units_per_minute, (int, long, float, 
            durationtools.Duration, list, tuple, type(None)))
        if isinstance(units_per_minute, (list, tuple)):
            assert len(units_per_minute) == 2
            assert all(
                isinstance(x, (int, long, float, durationtools.Duration)) 
                for x in units_per_minute)
            units_per_minute = tuple(sorted(units_per_minute))
        self._duration = duration
        self._textual_indication = textual_indication
        self._units_per_minute = units_per_minute

    ### SPECIAL METHODS ###

    def __add__(self, expr):
        r'''Adds tempo to `expr`.

        Returns new tempo.
        '''
        if isinstance(expr, type(self)):
            if self.is_imprecise or expr.is_imprecise:
                raise ImpreciseTempoError
            new_quarters_per_minute = \
                self.quarters_per_minute + expr.quarters_per_minute
            minimum_denominator = \
                min((self.duration.denominator, expr.duration.denominator))
            nonreduced_fraction = \
                mathtools.NonreducedFraction(new_quarters_per_minute / 4)
            nonreduced_fraction = \
                nonreduced_fraction.with_denominator(minimum_denominator)
            new_units_per_minute, new_duration_denominator = \
                nonreduced_fraction.pair
            new_duration = \
                durationtools.Duration(1, new_duration_denominator)
            new_tempo_indication = \
                type(self)(new_duration, new_units_per_minute)
            return new_tempo_indication

    def __copy__(self, *args):
        r'''Copies tempo.

        Returns new tempo.
        '''
        return type(self)(
            self.textual_indication, 
            self.duration, 
            self.units_per_minute,
            )

    def __div__(self, expr):
        r'''Divides tempo by `expr`.

        Returns new tempo.
        '''
        if isinstance(expr, type(self)):
            if self.is_imprecise or expr.is_imprecise:
                raise ImpreciseTempoError
            result = self.quarters_per_minute / expr.quarters_per_minute
            return durationtools.Multiplier(result)
        message = 'must be tempo indication.'
        raise TypeError(message)

    def __eq__(self, expr):
        r'''True when `expr` is a tempo with duration, textual indication
        and units-per-minute all equal to this tempo. Otherwise false.

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if self.duration == expr.duration:
                if self.textual_indication == expr.textual_indication:
                    if self.units_per_minute == expr.units_per_minute:
                        return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats tempo.

        Set `format_specification` to `''`', `'lilypond'` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ::

            >>> tempo = Tempo('Allegro', (1, 4), 84)
            >>> print format(tempo)
            indicatortools.Tempo(
                'Allegro',
                durationtools.Duration(1, 4),
                84
                )

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        elif format_specification == 'lilypond':
            return self._lilypond_format
        return str(self)

    def __lt__(self, arg):
        r'''True when `arg` is a tempo with quarters per minute greater than
        that of this tempo. Otherwise false.

        Returns boolean.
        '''
        assert isinstance(arg, type(self)), repr(arg)
        return self.quarters_per_minute < arg.quarters_per_minute

    def __mul__(self, multiplier):
        r'''Multiplies tempo by `multiplier`.

        ::

            >>> tempo = Tempo(Duration(1, 4), 84)
            >>> tempo * 2
            Tempo(Duration(1, 4), 168)

        Returns new tempo.
        '''
        if not isinstance(multiplier, (int, float, durationtools.Duration)):
            return
        if self.is_imprecise:
            raise ImpreciseTempoError
        new_units_per_minute = multiplier * self.units_per_minute
        new_duration = durationtools.Duration(self.duration)
        new_tempo = type(self)(new_duration, new_units_per_minute)
        return new_tempo

    def __rmul__(self, multiplier):
        r'''Multiplies `multiplier` by tempo.

        ::

            >>> tempo = Tempo(Duration(1, 4), 84)
            >>> 2 * tempo
            Tempo(Duration(1, 4), 168)

        Returns new tempo.
        '''
        if not isinstance(multiplier, (int, float, durationtools.Duration)):
            return
        if self.is_imprecise:
            raise ImpreciseTempoError
        new_units_per_minute = multiplier * self.units_per_minute
        new_duration = durationtools.Duration(self.duration)
        new_tempo = type(self)(new_duration, new_units_per_minute)
        return new_tempo

    def __str__(self):
        r'''String representation of tempo.

        ::

            >>> str(tempo)
            '4=84'

        Returns string.
        '''
        return self._equation or self.textual_indication

    def __sub__(self, expr):
        r'''Subtracts `expr` from tempo.

        ::

            >>> tempo - 20

        Returns new tempo.
        '''
        if isinstance(expr, type(self)):
            if self.is_imprecise or expr.is_imprecise:
                raise ImpreciseTempoError
            new_quarters_per_minute = \
                self.quarters_per_minute - expr.quarters_per_minute
            minimum_denominator = \
                min((self.duration.denominator, expr.duration.denominator))
            nonreduced_fraction = \
                mathtools.NonreducedFraction(new_quarters_per_minute / 4)
            nonreduced_fraction = \
                nonreduced_fraction.with_denominator(minimum_denominator)
            new_units_per_minute, new_duration_denominator = \
                nonreduced_fraction.pair
            new_duration = \
                durationtools.Duration(1, new_duration_denominator)
            new_tempo_indication = \
                type(self)(new_duration, new_units_per_minute)
            return new_tempo_indication

    ### PRIVATE PROPERTIES ###

    @property
    def _dotted(self):
        return self.duration.lilypond_duration_string

    @property
    def _equation(self):
        if self.duration is None:
            return
        if isinstance(self.units_per_minute, tuple):
            return '{}={}-{}'.format(
                self._dotted, 
                self.units_per_minute[0], 
                self.units_per_minute[1],
                )
        return '{}={}'.format(self._dotted, self.units_per_minute)

    @property
    def _lilypond_format(self):
        text, equation = None, None
        if self.textual_indication is not None:
            text = schemetools.Scheme.format_scheme_value(
                self.textual_indication)
        if self.duration is not None and self.units_per_minute is not None:
            equation = self._equation
        if text and equation:
            return r'\tempo {} {}'.format(text, equation)
        elif equation:
            return r'\tempo {}'.format(equation)
        elif text:
            return r'\tempo {}'.format(text)
        else:
            return r'\tempo \default'

    @property
    def _one_line_menuing_summary(self):
        return self._lilypond_format.lstrip(r'\tempo ')

    @property
    def _repr_specification(self):
        return self._storage_format_specification.__makenew__(
            is_indented=False,
            )

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = []
        if self.textual_indication:
            positional_argument_values.append(self.textual_indication)
        if self.duration:
            positional_argument_values.append(self.duration)
        if self.units_per_minute:
            positional_argument_values.append(self.units_per_minute)
        return systemtools.StorageFormatSpecification(
            self,
            positional_argument_values=positional_argument_values,
            )

    ### PUBLIC PROPERTIES ###


    @property
    def duration(self):
        r'''Duration of tempo.

        ::

            >>> tempo.duration
            Duration(1, 4)

        Returns duration.
        '''
        return self._duration

    @property
    def is_imprecise(self):
        r'''True if tempo is entirely textual or if tempo's
        units_per_minute is a range.

        ::

            >>> Tempo(Duration(1, 4), 60).is_imprecise
            False
            >>> Tempo('Langsam', 4, 60).is_imprecise
            False
            >>> Tempo('Langsam').is_imprecise
            True
            >>> Tempo('Langsam', 4, (35, 50)).is_imprecise
            True
            >>> Tempo(Duration(1, 4), (35, 50)).is_imprecise
            True

        Otherwise false:

        ::

            >>> Tempo(Duration(1, 4), 60).is_imprecise
            False

        Returns boolean.
        '''
        if self.duration is not None:
            if self.units_per_minute is not None:
                if not isinstance(self.units_per_minute, tuple):
                    return False
        return True

    @property
    def quarters_per_minute(self):
        r'''Quarters per minute of tempo.

        ::

            >>> tempo = Tempo(Duration(1, 8), 52)
            >>> tempo.quarters_per_minute
            Fraction(104, 1)

        Returns tuple when tempo `units_per_minute` is a range.

        Returns none when tempo is imprecise.

        Returns fraction otherwise.
        '''
        if self.is_imprecise:
            return None
        if isinstance(self.units_per_minute, tuple):
            low = durationtools.Duration(1, 4) / self.duration * \
                self.units_per_minute[0]
            high = durationtools.Duration(1, 4) / self.duration * \
                self.units_per_minute[1]
            return (low, high)
        result = durationtools.Duration(1, 4) / self.duration * \
            self.units_per_minute
        return fractions.Fraction(result)

    @property
    def textual_indication(self):
        r'''Optional textual indication of tempo.

        ::

            >>> tempo.textual_indication is None
            True

        Returns string or none.
        '''
        return self._textual_indication

    @property
    def units_per_minute(self):
        r'''Units per minute of tempo.

        ::

            >>> tempo.units_per_minute
            52

        Returns number.
        '''
        return self._units_per_minute

    ### PUBLIC METHODS ###

    def duration_to_milliseconds(self, duration):
        r'''Millisecond value of `duration` under a given tempo.

        ::

            >>> duration = (1, 4)
            >>> tempo = Tempo((1, 4), 60)
            >>> tempo.duration_to_milliseconds(duration)
            Duration(1000, 1)

        Returns duration.
        '''
        duration = durationtools.Duration(duration)
        whole_note_duration = 1000 \
            * durationtools.Multiplier(
                self.duration.denominator, 
                self.duration.numerator,
                ) \
            * durationtools.Multiplier(
                60, 
                self.units_per_minute,
                )
        return durationtools.Duration(duration * whole_note_duration)

    def is_tempo_token(self, expr):
        r'''True when `expr` can initialize tempo.

        ::

            >>> tempo = Tempo(Duration(1, 4), 72)
            >>> tempo.is_tempo_token((Duration(1, 4), 84))
            True

        Otherwise false:

        ::

            >>> tempo.is_tempo_token(84)
            False

        Returns boolean.
        '''
        try:
            tempo = type(self)(expr)
            return isinstance(tempo, type(self))
        except:
            return False

    def list_related_tempos(
        self, 
        maximum_numerator=None, 
        maximum_denominator=None,
        ):
        r'''Lists tempos related to this tempo.

        Returns list of tempo / ratio pairs.

        Each new tempo equals not less than half of this tempo
        and not more than twice this tempo.

        ..  container:: example

            Rewrites tempo ``58`` MM by ratios of the form ``n:d`` such that 
            ``1 <= n <= 8`` and ``1 <= d <= 8``:
            ...

            ::

                >>> tempo = Tempo(Duration(1, 4), 58)
                >>> pairs = tempo.list_related_tempos(
                ...     maximum_numerator=8, 
                ...     maximum_denominator=8,
                ...     )

            ::

                >>> for tempo, ratio in pairs:
                ...     string = '{!s}\t{!s}'.format(tempo, ratio)
                ...     print string
                4=29    1:2
                4=58    1:1
                4=87    3:2
                4=116   2:1

        ..  container:: example

            Rewrites tempo ``58`` MM by ratios of the form ``n:d`` such that
            ``1 <= n <= 30`` and ``1 <= d <= 30``:

            ::

                >>> tempo = Tempo(Duration(1, 4), 58)
                >>> pairs = tempo.list_related_tempos(
                ...     maximum_numerator=30, 
                ...     maximum_denominator=30,
                ...     )

            ::

                >>> for tempo, ratio in pairs:
                ...     string = '{!s}\t{!s}'.format(tempo, ratio)
                ...     print string
                ...
                4=30    15:29
                4=32    16:29
                4=34    17:29
                4=36    18:29
                4=38    19:29
                4=40    20:29
                4=42    21:29
                4=44    22:29
                4=46    23:29
                4=48    24:29
                4=50    25:29
                4=52    26:29
                4=54    27:29
                4=56    28:29
                4=58    1:1
                4=60    30:29

        Returns list.
        '''
        # assert integer tempo
        assert isinstance(self.units_per_minute, int), repr(self)
        # find divisors
        divisors = mathtools.divisors(self.units_per_minute)
        if maximum_denominator is not None:
            divisors = [x for x in divisors if x <= maximum_denominator]
        # make pairs
        pairs = []
        for divisor in divisors:
            start = int(math.ceil(divisor / 2.0))
            stop = 2 * divisor
            numerators = range(start, stop + 1)
            if maximum_numerator is not None:
                    numerators = [
                        x for x in numerators 
                        if x <= maximum_numerator
                        ]
        for numerator in numerators:
                ratio = mathtools.Ratio(numerator, divisor)
                multiplier = durationtools.Multiplier(*ratio)
                new_units_per_minute = multiplier * self.units_per_minute
                assert mathtools.is_integer_equivalent_expr(
                    new_units_per_minute)
                new_units_per_minute = int(new_units_per_minute)
                new_tempo = type(self)(self.duration, new_units_per_minute)
                pair = (new_tempo, ratio)
                if pair not in pairs:
                    pairs.append(pair)
        # sort pairs
        pairs.sort()
        # return pairs
        return pairs

    def rewrite_duration(self, duration, new_tempo):
        r'''Rewrite `duration` under `new_tempo`.

        Given `duration` governed by this tempo
        return new duration governed by `new_tempo`.

        Ensure that `duration` and new duration
        consume the same amount of time in seconds.

        ..  container:: example

            Consider the two tempo indications below. 
            
            ::

                >>> tempo = Tempo(Duration(1, 4), 60)
                >>> new_tempo = Tempo(Duration(1, 4), 90)

            `tempo` specifies quarter equal to ``60 MM``.

            `new_tempo` indication specifies quarter equal to ``90 MM``.

            `new_tempo` is ``3/2`` times as fast as `tempo`:

            ::

                >>> new_tempo / tempo
                Multiplier(3, 2)

            Note that a triplet eighth note under `tempo` equals a regular 
            eighth note under `new_tempo`:

            ::

                >>> tempo.rewrite_duration(Duration(1, 12), new_tempo)
                Duration(1, 8)

            And note that a regular eighth note under `tempo` equals a dotted
            sixteenth under `new_tempo`:

            ::

                >>> tempo.rewrite_duration(Duration(1, 8), new_tempo)
                Duration(3, 16)

        Returns duration.
        '''
        tempo_ratio = new_tempo / self
        new_duration = tempo_ratio * duration
        return new_duration



# -*- encoding: utf-8 -*-
from abjad import *


# TODO: change all calls to explicit keyword calls
def test_rhythmmakertools_DivisionIncisedNoteRhythmMaker___call___01():

    prefix_talea, prefix_lengths = [-8], [0, 1]
    suffix_talea, suffix_lengths = [-1], [1]
    talea_denominator = 32
    maker = rhythmmakertools.DivisionIncisedNoteRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator)

    divisions = [(5, 8), (5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 5/8
            c'2 ~
            c'16.
            r32
        }
        {
            \time 5/8
            r4
            c'4 ~
            c'16.
            r32
        }
        {
            \time 5/8
            c'2 ~
            c'16.
            r32
        }
        {
            \time 5/8
            r4
            c'4 ~
            c'16.
            r32
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/8
                c'2 ~
                c'16.
                r32
            }
            {
                \time 5/8
                r4
                c'4 ~
                c'16.
                r32
            }
            {
                \time 5/8
                c'2 ~
                c'16.
                r32
            }
            {
                \time 5/8
                r4
                c'4 ~
                c'16.
                r32
            }
        }
        '''
        )


# TODO: change all calls to explicit keyword calls
def test_rhythmmakertools_DivisionIncisedNoteRhythmMaker___call___02():

    prefix_talea, prefix_lengths = [-8], [1, 2, 3, 4]
    suffix_talea, suffix_lengths = [-1], [1]
    talea_denominator = 32
    maker = rhythmmakertools.DivisionIncisedNoteRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator)

    divisions = [(5, 8), (5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 5/8
            r4
            c'4 ~
            c'16.
            r32
        }
        {
            \time 5/8
            r4
            r4
            c'16.
            r32
        }
        {
            \time 5/8
            r4
            r4
            r8
        }
        {
            \time 5/8
            r4
            r4
            r8
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/8
                r4
                c'4 ~
                c'16.
                r32
            }
            {
                \time 5/8
                r4
                r4
                c'16.
                r32
            }
            {
                \time 5/8
                r4
                r4
                r8
            }
            {
                \time 5/8
                r4
                r4
                r8
            }
        }
        '''
        )


# TODO: change all calls to explicit keyword calls
def test_rhythmmakertools_DivisionIncisedNoteRhythmMaker___call___03():

    prefix_talea, prefix_lengths = [-1], [1]
    suffix_talea, suffix_lengths = [-8], [1, 2, 3]
    talea_denominator = 32
    maker = rhythmmakertools.DivisionIncisedNoteRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator)

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 5/8
            r32
            c'4 ~
            c'16.
            r4
        }
        {
            \time 5/8
            r32
            c'16.
            r4
            r4
        }
        {
            \time 5/8
            r32
            r4
            r4
            r16.
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/8
                r32
                c'4 ~
                c'16.
                r4
            }
            {
                \time 5/8
                r32
                c'16.
                r4
                r4
            }
            {
                \time 5/8
                r32
                r4
                r4
                r16.
            }
        }
        '''
        )


# TODO: change all calls to explicit keyword calls
def test_rhythmmakertools_DivisionIncisedNoteRhythmMaker___call___04():

    prefix_talea, prefix_lengths = [], [0]
    suffix_talea, suffix_lengths = [], [0]
    talea_denominator = 8
    maker = rhythmmakertools.DivisionIncisedNoteRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator)

    divisions = [(5, 8), (5, 8), (5, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 5/8
            c'2 ~
            c'8
        }
        {
            \time 5/8
            c'2 ~
            c'8
        }
        {
            \time 5/8
            c'2 ~
            c'8
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 5/8
                c'2 ~
                c'8
            }
            {
                \time 5/8
                c'2 ~
                c'8
            }
            {
                \time 5/8
                c'2 ~
                c'8
            }
        }
        '''
        )


# TODO: change all calls to explicit keyword calls
def test_rhythmmakertools_DivisionIncisedNoteRhythmMaker___call___05():

    prefix_talea, prefix_lengths = [-1], [1]
    suffix_talea, suffix_lengths = [-1], [1]
    talea_denominator = 8
    prolation_addenda = [1, 0, 3]
    maker = rhythmmakertools.DivisionIncisedNoteRhythmMaker(
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator,
        prolation_addenda = prolation_addenda)

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 4/8
            \times 4/5 {
                r8
                c'4.
                r8
            }
        }
        {
            \time 4/8
            {
                r8
                c'4
                r8
            }
        }
        {
            \time 4/8
            \times 4/7 {
                r8
                c'2 ~
                c'8
                r8
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 4/8
                \times 4/5 {
                    r8
                    c'4.
                    r8
                }
            }
            {
                \time 4/8
                {
                    r8
                    c'4
                    r8
                }
            }
            {
                \time 4/8
                \times 4/7 {
                    r8
                    c'2 ~
                    c'8
                    r8
                }
            }
        }
        '''
        )


# TODO: change all calls to explicit keyword calls
def test_rhythmmakertools_DivisionIncisedNoteRhythmMaker___call___06():

    maker = rhythmmakertools.DivisionIncisedNoteRhythmMaker(
        prefix_talea=[-1],
        prefix_lengths=[1],
        suffix_talea=[],
        suffix_lengths=[0],
        talea_denominator=32,
        prolation_addenda=[2, 0],
        secondary_divisions=[20])

    divisions = [(4, 8), (4, 8), (4, 8)]
    leaf_lists = maker(divisions)
    leaves = sequencetools.flatten_sequence(leaf_lists)

    staff = Staff(scoretools.make_spacer_skip_measures(divisions))
    scoretools.replace_contents_of_measures_in_expr(staff, leaves)
    scoretools.set_always_format_time_signature_of_measures_in_expr(staff)

    r'''
    \new Staff {
        {
            \time 4/8
            \times 8/9 {
                r32
                c'2 ~
                c'32
            }
        }
        {
            \time 4/8
            {
                r32
                c'16.
            }
            \tweak #'text #tuplet-number::calc-fraction-text
            \times 6/7 {
                r32
                c'4. ~
                c'32
            }
        }
        {
            \time 4/8
            {
                r32
                c'8..
            }
            \times 4/5 {
                r32
                c'4 ~
                c'32
            }
        }
    }
    '''

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 4/8
                \times 8/9 {
                    r32
                    c'2 ~
                    c'32
                }
            }
            {
                \time 4/8
                {
                    r32
                    c'16.
                }
                \tweak #'text #tuplet-number::calc-fraction-text
                \times 6/7 {
                    r32
                    c'4. ~
                    c'32
                }
            }
            {
                \time 4/8
                {
                    r32
                    c'8..
                }
                \times 4/5 {
                    r32
                    c'4 ~
                    c'32
                }
            }
        }
        '''
        )



# -*- encoding: utf-8 -*-
from abjad import *
import os


input_text = u"""Taraf'ın Türkiye'nin saygın gazetelerinden olduğunun anımsatıldığı dilekçede."""
other = u"ÜÖ"
even_more_other = u"Ü"

vowel_treatments = {
     'a':   (2, pitchtools.NamedPitch("G4")),
     'i':   (2, pitchtools.NamedPitch("E4")),
    u'ı':   (1, pitchtools.NamedPitch("E4")),
     'u':   (2, pitchtools.NamedPitch("C4")),
    u'ü':   (4, pitchtools.NamedPitch("C4")),
     'e':   (2, pitchtools.NamedPitch("A3")),
     'o':   (2, pitchtools.NamedPitch("F3")),
    u'ö':   (4, pitchtools.NamedPitch("F3")),
}

def letter_to_duration_numerator(letter):
    letter = letter.lower()
    try:
        treatment_pair = consonant_treatments[letter]
    except KeyError:
        treatment_pair = vowel_treatments[letter]
    duration_numerator = treatment_pair[0]
    return duration_numerator

def letter_to_pitch(letter):
    letter = letter.lower()
    try:
        treatment_pair = consonant_treatments[letter]
    except KeyError:
        treatment_pair = vowel_treatments[letter]
    pitch = treatment_pair[-1]
    return pitch

consonant_treatments = {
    'b':    (2, pitchtools.NamedPitch("G4")),
    'c':    (3, pitchtools.NamedPitch("G4")),
   u'ç':    (3, pitchtools.NamedPitch("G4")),
    'd':    (2, pitchtools.NamedPitch("G4")),
    'f':    (3, pitchtools.NamedPitch("E4")),
    'g':    (2, pitchtools.NamedPitch("E4")),
   u'ğ':    (0, pitchtools.NamedPitch("E4")),
    'h':    (3, pitchtools.NamedPitch("E4")),
    'k':    (1, pitchtools.NamedPitch("C4")),
    'l':    (4, pitchtools.NamedPitch("C4")),
    'm':    (4, pitchtools.NamedPitch("C4")),
    'n':    (4, pitchtools.NamedPitch("C4")),
    'p':    (2, pitchtools.NamedPitch("A3")),
    'r':    (2, pitchtools.NamedPitch("A3")),
    's':    (4, pitchtools.NamedPitch("A3")),
   u'ş':    (4, pitchtools.NamedPitch("A3")),
    't':    (1, pitchtools.NamedPitch("A3")),
    'v':    (3, pitchtools.NamedPitch("F3")),
    'y':    (3, pitchtools.NamedPitch("F3")),
    'z':    (4, pitchtools.NamedPitch("F3")),
}


def preprocess_input_text(input_text):
    input_text = input_text.lower()
    result = []
    alphabet = consonant_treatments.keys() + vowel_treatments.keys()
    permissible_characters = alphabet + [',', '.', ' ']
    for input_character in input_text:
        if input_character in permissible_characters:
            result.append(input_character)
    result = ''.join(result)
    return result

def count_vowels(word):
    vowel_count = 0
    for character in word:
        if character in vowel_treatments:
            vowel_count += 1
    return vowel_count

def count_consonants(word):
    consonant_count = 0
    for character in word:
        if character in consonant_treatments:
            consonant_count += 1
    return consonant_count

class WordAnalysis(object):

    def __init__(self, word):
        self.word = word
        self.word_length = len(word)
        self.vowel_count = 0
        self.consonant_count = 0
        self.vowel_pitches = []
        self.vowel_duration_numerators = []
        self.consonant_pitches = []
        self.consonant_duration_numerators = []
        self.analyze_word()

    def __repr__(self):
        return 'WordAnalysis({!r})'.format(self.word)

    def analyze_word(self):
        for character in self.word:
            if character in vowel_treatments:
                self.vowel_count += 1
                duration_numerator, pitch = vowel_treatments[character]
                self.vowel_duration_numerators.append(duration_numerator)
                self.vowel_pitches.append(pitch)
            elif character in consonant_treatments:
                duration_numerator, pitch = consonant_treatments[character]
                # account for silent g
                if duration_numerator:
                    self.consonant_count += 1
                    self.consonant_duration_numerators.append(duration_numerator)
                    self.consonant_pitches.append(pitch)
            else:
                raise ValueError('what is character {!r}?'.format(character))

def find_first_phrase(input_text):
    input_text = preprocess_input_text(input_text)
    first_phrase_index = min(input_text.find(','), input_text.find('.'))
    first_phrase = input_text[:first_phrase_index]
    return first_phrase

def analyze_phrase(phrase):
    phrase_words = phrase.split()
    word_analyses = []
    for phrase_word in phrase_words:
        word_analyses.append(WordAnalysis(phrase_word))
    return word_analyses

def make_lilypond_file(input_text, base_duration=Duration(1, 16)):
    first_phrase = find_first_phrase(input_text)
    word_analyses = analyze_phrase(first_phrase)
    upper_music, lower_music = make_music_for_word_analyses(word_analyses, base_duration)
    score_template = templatetools.GroupedStavesScoreTemplate(staff_count=2)
    score = score_template()
    staff_1 = scoretools.get_first_component_in_expr_with_name(score, 'Staff 1')
    staff_2 = scoretools.get_first_component_in_expr_with_name(score, 'Staff 2')
    indicatortools.Clef('percussion')(staff_1)
    indicatortools.Clef('percussion')(staff_2)
    staff_1.extend(upper_music)
    staff_2.extend(lower_music)

    override(staff_1).staff_symbol.line_count = 4
    override(staff_2).staff_symbol.line_count = 4

    lilypond_file = lilypondfiletools.make_basic_lilypond_file(score)
    format_score(score)
    format_lilypond_file(lilypond_file)
    return lilypond_file

def format_score(score):
    override(score).bar_line.transparent = True
    override(score).beam.positions = (-6, -6)
    override(score).spacing_spanner.strict_grace_spacing = True
    override(score).spacing_spanner.strict_note_spacing = True
    override(score).spacing_spanner.uniform_stretching = True
    override(score).span_bar.transparent = True
    override(score).stem.direction = 'down'
    override(score).time_signature.transparent = True
    contextualize(score).autoBeaming = False
    contextualize(score).proportionalNotationDuration = schemetools.SchemeMoment((1, 64))
    contextualize(score).tupletFullLength = True

    vector = layouttools.make_spacing_vector(0, 0, 6, 0)
    score[0].override.staff_grouper.staff_staff_spacing = vector

    return score

def format_lilypond_file(lilypond_file):
    lilypond_file.global_staff_size = 14
    lilypond_file.header_block.title = markuptools.Markup('Turkish text')
    lilypond_file.layout_block.indent = 0
    lilypond_file.layout_block.ragged_right = True
    vector = layouttools.make_spacing_vector(0, 0, 12, 0)
    lilypond_file.paper_block.markup_system_spacing = vector
    lilypond_file.paper_block.top_margin = 10
    return lilypond_file

def make_music_for_word_analyses(word_analyses, base_duration):
    vowel_counts = [x.vowel_count for x in word_analyses]
    upper_durations = [base_duration * x for x in vowel_counts]
    lower_durations = sequencetools.rotate_sequence(upper_durations, -1)
    upper_pitches = [x.vowel_pitches for x in word_analyses]
    lower_pitches = [x.consonant_pitches for x in word_analyses]
    upper_proportions = [x.vowel_duration_numerators for x in word_analyses]
    lower_proportions = [x.consonant_duration_numerators for x in word_analyses]
    upper_music = make_tuplets(upper_durations, upper_proportions, upper_pitches)
    lower_music = make_tuplets(lower_durations, lower_proportions, lower_pitches)
    return upper_music, lower_music

def make_tuplets(durations, proportions, pitch_lists):
    tuplets = []
    for duration, proportion, pitch_list in zip(
        durations, proportions, pitch_lists):
        tuplet = Tuplet.from_duration_and_ratio(duration, proportion)
        spannertools.Beam(tuplet)
        for leaf, pitch in zip(tuplet.leaves, pitch_list):
            leaf.written_pitch = pitch
        tuplets.append(tuplet)
    return tuplets


if __name__ == '__main__':
    os.system('clear')
    lilypond_file = make_lilypond_file(input_text)
    show(lilypond_file)



# -*- encoding: utf-8 -*-
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.rhythmmakertools.BurnishedRhythmMaker \
	import BurnishedRhythmMaker


class DivisionBurnishedTaleaRhythmMaker(BurnishedRhythmMaker):
    r'''Division-burnished talea rhythm-maker:

    ::

        >>> maker = rhythmmakertools.DivisionBurnishedTaleaRhythmMaker(
        ...     talea=[1, 1, 2, 4],
        ...     talea_denominator=16,
        ...     prolation_addenda=[0, 3],
        ...     lefts=[-1],
        ...     middles=[0],
        ...     rights=[-1],
        ...     left_lengths=[1],
        ...     right_lengths=[1],
        ...     secondary_divisions=[14])

    Configure at instantiation and then call on any sequence of divisions:

    ::

        >>> divisions = [(5, 8), (5, 8)]
        >>> music = maker(divisions)
        >>> music = sequencetools.flatten_sequence(music)
        >>> measures = \
        ...     scoretools.make_spacer_skip_measures(
        ...     divisions)
        >>> staff = scoretools.RhythmicStaff(measures)
        >>> measures = scoretools.replace_contents_of_measures_in_expr(
        ...     staff, music)

    ::

        >>> show(staff) # doctest: +SKIP

    Usage follows the two-step instantiate-then-call pattern shown here.
    '''

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats division-burnished talea rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ::

            >>> print format(maker)
            rhythmmakertools.DivisionBurnishedTaleaRhythmMaker(
                talea=[1, 1, 2, 4],
                talea_denominator=16,
                prolation_addenda=[0, 3],
                lefts=[-1],
                middles=[0],
                rights=[-1],
                left_lengths=[1],
                right_lengths=[1],
                secondary_divisions=[14],
                beam_each_cell=False,
                beam_cells_together=False,
                decrease_durations_monotonically=True,
                tie_split_notes=False,
                tie_rests=False,
                )

        Returns string.
        '''
        superclass = super(DivisionBurnishedTaleaRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __makenew__(self, *args, **kwargs):
        r'''Creates new rhythm-maker with `kwargs`.

        ::

            >>> print format(maker)
            rhythmmakertools.DivisionBurnishedTaleaRhythmMaker(
                talea=[1, 1, 2, 4],
                talea_denominator=16,
                prolation_addenda=[0, 3],
                lefts=[-1],
                middles=[0],
                rights=[-1],
                left_lengths=[1],
                right_lengths=[1],
                secondary_divisions=[14],
                beam_each_cell=False,
                beam_cells_together=False,
                decrease_durations_monotonically=True,
                tie_split_notes=False,
                tie_rests=False,
                )

        ::

            >>> new_maker = new(maker, talea=[1, 1, 2])

        ::

            >>> print format(new_maker)
            rhythmmakertools.DivisionBurnishedTaleaRhythmMaker(
                talea=[1, 1, 2],
                talea_denominator=16,
                prolation_addenda=[0, 3],
                lefts=[-1],
                middles=[0],
                rights=[-1],
                left_lengths=[1],
                right_lengths=[1],
                secondary_divisions=[14],
                beam_each_cell=False,
                beam_cells_together=False,
                decrease_durations_monotonically=True,
                tie_split_notes=False,
                tie_rests=False,
                )

        ::

            >>> divisions = [(5, 8), (5, 8)]
            >>> music = new_maker(divisions)
            >>> music = sequencetools.flatten_sequence(music)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new division-burnished talea rhythm-maker.
        '''
        return BurnishedRhythmMaker.__makenew__(self, *args, **kwargs)

    ### PRIVATE METHODS ###

    def _burnish_division_parts(self, divisions, quintuplet):
        lefts, middles, rights, left_lengths, right_lengths=quintuplet
        lefts_index, rights_index = 0, 0
        burnished_divisions = []
        for division_index, division in enumerate(divisions):
            left_length = left_lengths[division_index]
            left = lefts[lefts_index:lefts_index+left_length]
            lefts_index += left_length
            right_length = right_lengths[division_index]
            right = rights[rights_index:rights_index+right_length]
            rights_index += right_length
            available_left_length = len(division)
            left_length = min([left_length, available_left_length])
            available_right_length = len(division) - left_length
            right_length = min([right_length, available_right_length])
            middle_length = len(division) - left_length - right_length

            left = left[:left_length]
            middle = middle_length * [middles[division_index]]
            right = right[:right_length]
            left_part, middle_part, right_part = \
                sequencetools.partition_sequence_by_counts(
                division, 
                [left_length, middle_length, right_length], 
                cyclic=False, 
                overhang=False,
                )
            left_part = self._burnish_division_part(left_part, left)
            middle_part = self._burnish_division_part(middle_part, middle)
            right_part = self._burnish_division_part(right_part, right)
            burnished_division = left_part + middle_part + right_part
            burnished_divisions.append(burnished_division)
        unburnished_weights = [mathtools.weight(x) for x in divisions]
        burnished_weights = [mathtools.weight(x) for x in burnished_divisions]
        assert burnished_weights == unburnished_weights
        return burnished_divisions

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses division-burnished talea rhythm-maker.

        Nonreversed output:

        ::

            >>> print format(maker)
            rhythmmakertools.DivisionBurnishedTaleaRhythmMaker(
                talea=[1, 1, 2, 4],
                talea_denominator=16,
                prolation_addenda=[0, 3],
                lefts=[-1],
                middles=[0],
                rights=[-1],
                left_lengths=[1],
                right_lengths=[1],
                secondary_divisions=[14],
                beam_each_cell=False,
                beam_cells_together=False,
                decrease_durations_monotonically=True,
                tie_split_notes=False,
                tie_rests=False,
                )

        ::

            >>> divisions = [(5, 8), (5, 8)]
            >>> music = maker(divisions)
            >>> music = sequencetools.flatten_sequence(music)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Reversed output:

        ::

            >>> reversed_maker = maker.reverse()

        ::

            >>> print format(reversed_maker)
            rhythmmakertools.DivisionBurnishedTaleaRhythmMaker(
                talea=[4, 2, 1, 1],
                talea_denominator=16,
                prolation_addenda=[3, 0],
                lefts=[-1],
                middles=[0],
                rights=[-1],
                left_lengths=[1],
                right_lengths=[1],
                secondary_divisions=[14],
                beam_each_cell=False,
                beam_cells_together=False,
                decrease_durations_monotonically=False,
                tie_split_notes=False,
                tie_rests=False,
                )

        ::

            >>> divisions = [(5, 8), (5, 8)]
            >>> music = reversed_maker(divisions)
            >>> music = sequencetools.flatten_sequence(music)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new division-burnished talea rhythm-maker.
        '''
        return BurnishedRhythmMaker.reverse(self)



# -*- encoding: utf-8 -*-
from abjad import *


def test_selectiontools_ContiguousSelection_partition_by_durations_exactly_01():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    tempo = Tempo(Duration(1, 4), 60)
    attach(tempo, staff, scope=Staff)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \tempo 4=60
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )


    leaves = staff.select_leaves()
    parts = leaves.partition_by_durations_exactly(
        [1.5],
        cyclic=True,
        in_seconds=True,
        overhang=False
        )

    r'''
    [
        [Note(c'', 8), Note(b', 8), Note(a', 8)],
        [Note(g', 8), Note(f', 8), Note(e', 8)]
    ]
    '''

    assert len(parts) == 2
    assert parts[0] == list(staff.select_leaves()[:3])
    assert parts[1] == list(staff.select_leaves()[3:6])


def test_selectiontools_ContiguousSelection_partition_by_durations_exactly_02():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    tempo = Tempo(Duration(1, 4), 60)
    attach(tempo, staff, scope=Staff)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \tempo 4=60
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    leaves = staff.select_leaves()
    parts = leaves.partition_by_durations_exactly(
        [1.5],
        cyclic=True,
        in_seconds=True,
        overhang=True,
        )

    r'''
    [
        [Note(c'', 8), Note(b', 8), Note(a', 8)],
        [Note(g', 8), Note(f', 8), Note(e', 8)],
        [Note(d', 8), Note(c', 8)]
    ]
    '''

    assert len(parts) == 3
    assert parts[0] == list(staff.select_leaves()[:3])
    assert parts[1] == list(staff.select_leaves()[3:6])
    assert parts[2] == list(staff.select_leaves()[6:8])


def test_selectiontools_ContiguousSelection_partition_by_durations_exactly_03():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    leaves = staff.select_leaves()
    parts = leaves.partition_by_durations_exactly(
        [Duration(3, 8)],
        cyclic=True,
        in_seconds=False,
        overhang=True,
        )

    r'''
    [
        [Note(c'', 8), Note(b', 8), Note(a', 8)]
        [Note(g', 8), Note(f', 8), Note(e', 8)]
        [Note(d', 8), Note(c', 8)]
    ]
    '''

    assert len(parts) == 3
    assert parts[0] == list(staff.select_leaves()[:3])
    assert parts[1] == list(staff.select_leaves()[3:6])
    assert parts[2] == list(staff.select_leaves()[6:8])


def test_selectiontools_ContiguousSelection_partition_by_durations_exactly_04():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")
    tempo = Tempo(Duration(1, 4), 60)
    attach(tempo, staff, scope=Staff)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            \tempo 4=60
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    leaves = staff.select_leaves()
    parts = leaves.partition_by_durations_exactly(
        [1.5],
        cyclic=False,
        in_seconds=True,
        overhang=False,
        )

    "[[Note(c'', 8), Note(b', 8), Note(a', 8)]]"

    assert len(parts) == 1
    assert parts[0] == list(staff.select_leaves()[:3])


def test_selectiontools_ContiguousSelection_partition_by_durations_exactly_05():

    staff = Staff("abj: | 2/8 c'8 d'8 || 2/8 e'8 f'8 |"
        "| 2/8 g'8 a'8 || 2/8 b'8 c''8 |")

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                e'8
                f'8
            }
            {
                g'8
                a'8
            }
            {
                b'8
                c''8
            }
        }
        '''
        )

    leaves = staff.select_leaves()
    parts = leaves.partition_by_durations_exactly(
        [Duration(3, 8)],
        cyclic=False,
        in_seconds=False,
        overhang=False,
        )

    "[[Note(c'', 8), Note(b', 8), Note(a', 8)]]"

    assert len(parts) == 1
    assert parts[0] == list(staff.select_leaves()[:3])



# -*- encoding: utf-8 -*-
import math
from abjad.tools import scoretools
from abjad.tools import durationtools
from abjad.tools import scoretools
from abjad.tools import mathtools
from abjad.tools import scoretools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker


class EqualDivisionRhythmMaker(RhythmMaker):
    r'''Equal division rhythm-maker:

    ::

        >>> maker = rhythmmakertools.EqualDivisionRhythmMaker(leaf_count=4)

    Configure at initialization and then call on any series of divisions:

    ::

        >>> divisions = [(1, 2), (3, 8), (5, 16)]
        >>> tuplet_lists = maker(divisions)
        >>> music = sequencetools.flatten_sequence(tuplet_lists)
        >>> measures = \
        ...     scoretools.make_spacer_skip_measures(
        ...     divisions)
        >>> staff = scoretools.RhythmicStaff(measures)
        >>> measures = scoretools.replace_contents_of_measures_in_expr(
        ...     staff, music)

    ::

        >>> show(staff) # doctest: +SKIP

    Usage follows the two-step instantiate-then-call pattern shown here.
    '''

    ### INITIALIZER ###

    def __init__(
        self,
        leaf_count=1,
        is_diminution=True,
        beam_each_cell=True,
        beam_cells_together=False,
        ):
        assert mathtools.is_integer_equivalent_expr(leaf_count)
        RhythmMaker.__init__(self,
            beam_each_cell=beam_each_cell,
            beam_cells_together=beam_cells_together
            )
        leaf_count = int(leaf_count)
        self._leaf_count = leaf_count
        self._is_diminution = is_diminution

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        r'''Calls equal-division rhythm-maker on `divisions`.

        Returns list of tuplet lists.
        '''
        result = []
        for division in divisions:
            tuplet = self._make_tuplet(division)
            result.append([tuplet])
        return result

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats equal division rhythm-maker.

        Set `format_specification` to `''` or `'storage'`.

        ::

            >>> print format(maker)
            rhythmmakertools.EqualDivisionRhythmMaker(
                leaf_count=4,
                is_diminution=True,
                beam_each_cell=True,
                beam_cells_together=False,
                )

        Returns string.
        '''
        superclass = super(EqualDivisionRhythmMaker, self)
        return superclass.__format__(format_specification=format_specification)

    def __makenew__(self, *args, **kwargs):
        r'''Creates new equal-division rhythm-maker with `kwargs`.

        ::

            >>> new_maker = new(maker, is_diminution=False)

        ::

            >>> print format(new_maker)
            rhythmmakertools.EqualDivisionRhythmMaker(
                leaf_count=4,
                is_diminution=False,
                beam_each_cell=True,
                beam_cells_together=False,
                )

        ::

            >>> divisions = [(1, 2), (3, 8), (5, 16)]
            >>> tuplet_lists = new_maker(divisions)
            >>> music = sequencetools.flatten_sequence(tuplet_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Returns new equal-division rhythm-maker.
        '''
        return RhythmMaker.__makenew__(self, *args, **kwargs)

    ### PRIVATE METHODS ###

    def _make_tuplet(self, division):
        numerator, talea_denominator = division
        division_duration = durationtools.Duration(division)
        ratio = self.leaf_count * [1]
        tuplet = scoretools.Tuplet.from_duration_and_ratio(
            division_duration,
            ratio,
            avoid_dots=True,
            is_diminution=self.is_diminution,
            )
        return tuplet

    ### PUBLIC PROPERTIES ###

    @property
    def is_diminution(self):
        r'''True when output tuplets should be diminuted.

        False when output tuplets should be augmented:

        ::

            >>> maker.is_diminution
            True

        Returns boolean.
        '''
        return self._is_diminution

    @property
    def leaf_count(self):
        r'''Number of leaves per division:

        ::

            >>> maker.leaf_count
            4

        Returns positive integer.
        '''
        return self._leaf_count

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''Reverses equal-division rhythm-maker.

        ::

            >>> reversed_maker = maker.reverse()

        ::

            >>> print format(reversed_maker)
            rhythmmakertools.EqualDivisionRhythmMaker(
                leaf_count=4,
                is_diminution=True,
                beam_each_cell=True,
                beam_cells_together=False,
                )

        ::

            >>> divisions = [(1, 2), (3, 8), (5, 16)]
            >>> tuplet_lists = reversed_maker(divisions)
            >>> music = sequencetools.flatten_sequence(tuplet_lists)
            >>> measures = \
            ...     scoretools.make_spacer_skip_measures(
            ...     divisions)
            >>> staff = scoretools.RhythmicStaff(measures)
            >>> measures = scoretools.replace_contents_of_measures_in_expr(
            ...     staff, music)

        ::

            >>> show(staff) # doctest: +SKIP

        Defined equal to copy of maker.

        Returns new equal-division rhythm-maker.
        '''
        return RhythmMaker.__makenew__(self)



# -*- encoding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHeadInventory_remove_01():
    '''Remove note head from chord.
    '''

    chord = Chord("<ef' cs'' f''>4")
    note_head = chord.note_heads[1]
    chord.note_heads.remove(note_head)

    assert note_head._client is None
    assert format(chord) == "<ef' f''>4"



# -*- encoding: utf-8 -*-
import pytest
from abjad import *


def test_mathtools_cumulative_sums_zero_01():
    r'''Returns list of the cumulative sums of the integer elements in input.
    '''

    assert mathtools.cumulative_sums([1, 2, 3]) == [0, 1, 3, 6]
    assert mathtools.cumulative_sums([10, -9, -8]) == [0, 10, 1, -7]
    assert mathtools.cumulative_sums([0, 0, 0, 5]) == [0, 0, 0, 0, 5]
    assert mathtools.cumulative_sums([-10, 10, -10, 10]) == \
        [0, -10, 0, -10, 0]



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_iterate_sequence_cyclically_01():
    r'''Defaults step to 1 and start to 0.
    '''

    sequence_1 = [1, 2, 3, 4, 5, 6, 7]
    sequence_2 = list(sequencetools.iterate_sequence_cyclically(sequence_1, length=20))

    assert sequence_2 == [1, 2, 3, 4, 5, 6, 7, 1, 2, 3, 4, 5, 6, 7, 1, 2, 3, 4, 5, 6]


def test_sequencetools_iterate_sequence_cyclically_02():
    r'''Step can be greater than 1.
    '''

    sequence_1 = [1, 2, 3, 4, 5, 6, 7]
    step = list(sequencetools.iterate_sequence_cyclically(sequence_1, 2, length=20))

    assert step == [1, 3, 5, 7, 2, 4, 6, 1, 3, 5, 7, 2, 4, 6, 1, 3, 5, 7, 2, 4]


def test_sequencetools_iterate_sequence_cyclically_03():
    r'''Start can be greater than 0.
    '''

    sequence_1 = [1, 2, 3, 4, 5, 6, 7]
    sequence_2 = list(sequencetools.iterate_sequence_cyclically(sequence_1, 2, 3, length=20))

    assert sequence_2 == [4, 6, 1, 3, 5, 7, 2, 4, 6, 1, 3, 5, 7, 2, 4, 6, 1, 3, 5, 7]


def test_sequencetools_iterate_sequence_cyclically_04():
    r'''Step can be negative.
    '''

    sequence_1 = [1, 2, 3, 4, 5, 6, 7]
    step = list(sequencetools.iterate_sequence_cyclically(sequence_1, -2, 5, length=20))

    assert step == [6, 4, 2, 7, 5, 3, 1, 6, 4, 2, 7, 5, 3, 1, 6, 4, 2, 7, 5, 3]


def test_sequencetools_iterate_sequence_cyclically_05():
    r'''Works on generator input.
    '''

    sequence_2 = list(sequencetools.iterate_sequence_cyclically(xrange(1, 8), length=20))
    assert sequence_2 == [1, 2, 3, 4, 5, 6, 7, 1, 2, 3, 4, 5, 6, 7, 1, 2, 3, 4, 5, 6]

    sequence_2 = list(sequencetools.iterate_sequence_cyclically(xrange(1, 8), 2, length=20))
    assert sequence_2 == [1, 3, 5, 7, 2, 4, 6, 1, 3, 5, 7, 2, 4, 6, 1, 3, 5, 7, 2, 4]

    sequence_2 = list(sequencetools.iterate_sequence_cyclically(xrange(1, 8), 2, 3, length=20))
    assert sequence_2 == [4, 6, 1, 3, 5, 7, 2, 4, 6, 1, 3, 5, 7, 2, 4, 6, 1, 3, 5, 7]

    sequence_2 = list(sequencetools.iterate_sequence_cyclically(xrange(1, 8), step=-2, start=5, length=20))
    assert sequence_2 == [6, 4, 2, 7, 5, 3, 1, 6, 4, 2, 7, 5, 3, 1, 6, 4, 2, 7, 5, 3]



# -*- encoding: utf-8 -*-
import os
import types
from abjad.tools.documentationtools.Documenter import Documenter


class ToolsPackageDocumenter(Documenter):
    r'''Generates an index for every tools package.
    '''

    ### INITIALIZER ###

    def __init__(
        self,
        object_=None,
        ignored_directory_names=(),
        prefix='abjad.tools.',
        ):
        if object_ is None:
            return
        assert isinstance(object_, types.ModuleType)
        Documenter.__init__(self, object_, prefix=prefix)
        self._ignored_directory_names = ignored_directory_names
        self._examine_tools_package()

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Calls tools package documenter.
        
        Generates documentation:

        ::

            >>> module = scoretools
            >>> documenter = documentationtools.ToolsPackageDocumenter(
            ...     scoretools)
            >>> restructured_text = documenter()

        Returns string.
        '''
        from abjad.tools import documentationtools
        document = documentationtools.ReSTDocument()
        document.append(documentationtools.ReSTHeading(
            level=2,
            text=self._shrink_module_name(self.module_name)
            ))
        document.append(documentationtools.ReSTAutodocDirective(
            argument=self.module_name,
            directive='automodule',
            ))
        html_only = documentationtools.ReSTOnlyDirective(argument='html')
        if self.abstract_class_documenters:
            html_only.extend(self._build_autosummary_section(
                'Abstract classes',
                self.abstract_class_documenters,
                ))
        if self.concrete_class_documenters:
            html_only.extend(self._build_autosummary_section(
                'Concrete classes',
                self.concrete_class_documenters,
                ))
        if self.function_documenters:
            html_only.extend(self._build_autosummary_section(
                'Functions',
                self.function_documenters,
                ))
        document.append(html_only)
        return document.rest_format

    ### PRIVATE METHODS ###

    def _build_autosummary_section(self, banner, documenters):
        from abjad.tools import documentationtools
        result = []
        heading = documentationtools.ReSTHeading(
            level=3,
            text=banner,
            )
        result.append(heading)
        autosummary = documentationtools.ReSTAutosummaryDirective()
        for documenter in documenters:
            autosummary.append('~{}'.format(documenter.module_name))
        result.append(autosummary)
        return result

    def _examine_tools_package(self):
        from abjad.tools import documentationtools
        code_root = self.object_.__path__[0]
        root_package_name = self.prefix.split('.')[0]
        crawler = documentationtools.ModuleCrawler(
            code_root,
            root_package_name=root_package_name,
            ignored_directory_names=self.ignored_directory_names,
            )
        abstract_class_documenters = []
        concrete_class_documenters = []
        function_documenters = []
        for module in crawler:
            obj_name = module.__name__.split('.')[-1]
            if not hasattr(module, obj_name) or obj_name.startswith('_'):
                continue
            obj = getattr(module, obj_name)
            if isinstance(obj, types.TypeType):
                documenter = documentationtools.ClassDocumenter(
                    obj,
                    prefix=self.prefix,
                    )
                if documenter.is_abstract:
                    abstract_class_documenters.append(documenter)
                else:
                    concrete_class_documenters.append(documenter)
            elif isinstance(obj, types.FunctionType):
                documenter = documentationtools.FunctionDocumenter(
                    obj,
                    prefix=self.prefix,
                    )
                function_documenters.append(documenter)
        self._abstract_class_documenters = tuple(sorted(
            abstract_class_documenters,
            key=lambda x: x.module_name))
        self._concrete_class_documenters = tuple(sorted(
            concrete_class_documenters,
            key=lambda x: x.module_name))
        self._function_documenters = tuple(sorted(
            function_documenters,
            key=lambda x: x.module_name))
        self._documentation_section = getattr(self.object_,
            '_documentation_section', None)

    ### PUBLIC METHODS ###

    def create_api_toc_section(self):
        r'''Creates a TOC section to be included in the master API index.

        ::

            >>> module = scoretools
            >>> documenter = documentationtools.ToolsPackageDocumenter(module)
            >>> result = documenter.create_api_toc_section()

        Returns list.
        '''
        from abjad.tools import documentationtools
        result = []
        heading = documentationtools.ReSTHeading(
            level=2,
            text=':py:mod:`{} <{}>`'.format(
                self._shrink_module_name(self.module_name), self.module_name)
            )
        result.append(heading)

        only_html = documentationtools.ReSTOnlyDirective(argument='html')
        only_latex = documentationtools.ReSTOnlyDirective(argument='latex')

        hidden_toc = documentationtools.ReSTTOCDirective(
            options={
                'hidden': True,
                'maxdepth': 1,
                },
            )
        index_path = '/'.join(self.module_name.split('.')[1:] + ['index'])
        hidden_toc.append(index_path)
        only_html.append(hidden_toc)

        def module_name_to_toc_entry(module_name):
            return '/'.join(module_name.split('.')[1:-1])

        def make_subsection(banner, documenters, only_html, only_latex):
            only_latex.append(documentationtools.ReSTHeading(
                level=3,
                text=banner
                ))
            toc_html = documentationtools.ReSTTOCDirective(
                options={'maxdepth': 1},
                )
            toc_latex = documentationtools.ReSTTOCDirective()
            for documenter in documenters:
                toc_entry = module_name_to_toc_entry(
                    documenter.module_name)
                toc_html.append(toc_entry)
                toc_latex.append(toc_entry)
            only_html.append(toc_html)
            only_latex.append(toc_latex)

        if self.abstract_class_documenters:
            make_subsection(
                'Abstract classes',
                self.abstract_class_documenters,
                only_html,
                only_latex,
                )
        if self.concrete_class_documenters:
            make_subsection(
                'Concrete classes',
                self.concrete_class_documenters,
                only_html,
                only_latex,
                )
        if self.function_documenters:
            make_subsection(
                'Functions',
                self.function_documenters,
                only_html,
                only_latex,
                )
        result.extend((only_html, only_latex))
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def abstract_class_documenters(self):
        r'''Abstract class documenters.
        '''
        return self._abstract_class_documenters

    @property
    def all_documenters(self):
        r'''All documenters.
        '''
        return self.abstract_class_documenters + \
            self.concrete_class_documenters + \
            self.function_documenters

    @property
    def concrete_class_documenters(self):
        r'''Concrete class documenters.
        '''
        return self._concrete_class_documenters

    @property
    def documentation_section(self):
        r'''Documentation section.
        '''
        return self._documentation_section

    @property
    def function_documenters(self):
        r'''Function documenters.
        '''
        return self._function_documenters

    @property
    def ignored_directory_names(self):
        r'''Ignored directory names.
        '''
        return self._ignored_directory_names

    @property
    def module_name(self):
        r'''Module name.

        Returns string.
        '''
        return self.object_.__name__



# -*- encoding: utf-8 -*-
from abjad import *


def test_labeltools_color_measures_with_non_power_of_two_denominators_in_expr_01():

    staff = Staff(2 * Measure((2, 8), "c'8 d'8"))
    scoretools.scale_measure_denominator_and_adjust_measure_contents(
        staff[1], 3)

    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \time 3/12
                \scaleDurations #'(2 . 3) {
                    c'8.
                    d'8.
                }
            }
        }
        '''
        )

    labeltools.color_measures_with_non_power_of_two_denominators_in_expr(
        staff, 'red')


    assert systemtools.TestManager.compare(
        staff,
        r'''
        \new Staff {
            {
                \time 2/8
                c'8
                d'8
            }
            {
                \override Beam #'color = #red
                \override Dots #'color = #red
                \override NoteHead #'color = #red
                \override Staff.TimeSignature #'color = #red
                \override Stem #'color = #red
                \time 3/12
                \scaleDurations #'(2 . 3) {
                    c'8.
                    d'8.
                }
                \revert Beam #'color
                \revert Dots #'color
                \revert NoteHead #'color
                \revert Staff.TimeSignature #'color
                \revert Stem #'color
            }
        }
        '''
        )

    assert inspect(staff).is_well_formed()



# -*- encoding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import schemetools
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import contextualize


def make_floating_time_signature_lilypond_file(music=None):
    r'''Make floating time signature LilyPond file from `music`.

    Function creates a basic LilyPond file.

    Function then applies many layout settings.

    View source here for the complete inventory of settings applied.

    Returns LilyPond file object.
    '''
    from abjad.tools import layouttools
    from abjad.tools import lilypondfiletools

    lilypond_file = lilypondfiletools.make_basic_lilypond_file(music=music)

    lilypond_file.default_paper_size = 'letter', 'portrait'
    lilypond_file.global_staff_size = 12

    lilypond_file.paper_block.left_margin = 20
    lilypond_file.paper_block.system_system_spacing = \
        layouttools.make_spacing_vector(0, 0, 12, 0)

    lilypond_file.layout_block.indent = 0
    lilypond_file.layout_block.ragged_right = True
    lilypond_file.layout_block.append(
        indicatortools.LilyPondCommand('accidentalStyle forget'))

    time_signature_context_block = \
        lilypondfiletools.make_time_signature_context_block(
        font_size=1, padding=6)
    lilypond_file.layout_block.context_blocks.append(
        time_signature_context_block)

    context_block = lilypondfiletools.ContextBlock()
    lilypond_file.layout_block.context_blocks.append(context_block)
    lilypond_file.score = context_block
    context_block.context_name = 'Score'
    context_block.accepts.append('TimeSignatureContext')
    context_block.engraver_removals.append('Bar_number_engraver')
    override(context_block).beam.breakable = True
    override(context_block).spacing_spanner.strict_grace_spacing = True
    override(context_block).spacing_spanner.strict_note_spacing = True
    override(context_block).spacing_spanner.uniform_stretching = True
    override(context_block).tuplet_bracket.bracket_visibility = True
    override(context_block).tuplet_bracket.padding = 2
    override(context_block).tuplet_bracket.springs_and_rods = \
        schemetools.Scheme('ly:spanner::set-spacing-rods')
    override(context_block).tuplet_bracket.minimum_length = 3
    override(context_block).tuplet_number.text = \
        schemetools.Scheme('tuplet-number::calc-fraction-text')
    contextualize(context_block).autoBeaming = False
    contextualize(context_block).proportionalNotationDuration = \
        schemetools.SchemeMoment((1, 32))
    contextualize(context_block).tupletFullLength = True

    # provided as a stub position for user customization
    context_block = lilypondfiletools.ContextBlock()
    lilypond_file.layout_block.context_blocks.append(context_block)
    context_block.context_name = 'StaffGroup'

    context_block = lilypondfiletools.ContextBlock()
    lilypond_file.layout_block.context_blocks.append(context_block)
    context_block.context_name = 'Staff'
    context_block.engraver_removals.append('Time_signature_engraver')

    context_block = lilypondfiletools.ContextBlock()
    lilypond_file.layout_block.context_blocks.append(context_block)
    context_block.context_name = 'RhythmicStaff'
    context_block.engraver_removals.append('Time_signature_engraver')

    return lilypond_file



# -*- encoding: utf-8 -*-
import unicodedata


def strip_diacritics_from_binary_string(binary_string):
    r'''Strip diacritics from `binary_string`:

    ::

        >>> binary_string = 'Dvo\xc5\x99\xc3\xa1k'

    ::

        >>> print binary_string
        Dvořák

    ::

        >>> stringtools.strip_diacritics_from_binary_string(binary_string)
        'Dvorak'

    Returns ASCII string.
    '''

    unicode_string = unicode(binary_string, 'utf-8')
    normalized_unicode_string = unicodedata.normalize('NFKD', unicode_string)
    ascii_string = normalized_unicode_string.encode('ascii', 'ignore')

    return ascii_string



# -*- encoding: utf-8 -*-
import copy
from abjad.tools.abctools.AbjadObject import AbjadObject


class PayloadTree(AbjadObject):
    r'''A payload tree.
    
    Abjad data structure to work with a sequence whose elements have been
    grouped into arbitrarily many levels of containment.

    Here is a tree:

    ::

        >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
        >>> tree = datastructuretools.PayloadTree(sequence)

    ::

        >>> tree
        PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])

    ::

        >>> tree.parent is None
        True

    ::

        >>> for x in tree.children:
        ...     x
        PayloadTree([0, 1])
        PayloadTree([2, 3])
        PayloadTree([4, 5])
        PayloadTree([6, 7])

    ::

        >>> tree.depth
        3

    Here's an internal node:

    ::

        >>> tree[2]
        PayloadTree([4, 5])

    ::

        >>> tree[2].parent
        PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])

    ::

        >>> tree[2].children
        (PayloadTree(4), PayloadTree(5))

    ::

        >>> tree[2].depth
        2

    ::

        >>> tree[2].level
        1

    Here's a leaf node:

    ::

        >>> tree[2][0]
        PayloadTree(4)

    ::

        >>> tree[2][0].parent
        PayloadTree([4, 5])

    ::

        >>> tree[2][0].children
        ()

    ::

        >>> tree[2][0].depth
        1

    ::

        >>> tree[2][0].level
        2

    ::

        >>> tree[2][0].position
        (2, 0)

    ::

        >>> tree[2][0].payload
        4

    Only leaf nodes carry payload. Internal nodes carry no payload.

    Negative levels are available to work with trees bottom-up 
    instead of top-down.

    Trees do not yet implement append or extend methods.
    '''

    ### INITIALIZER ###

    def __init__(self, expr=None):
        if isinstance(expr, type(self)):
            expr = expr.to_nested_lists()
        self._children = self._initialize_children_list()
        self.parent = None
        try:
            self._payload = None
            for element in expr:
                child = type(self)(element)
                self._children.append(child)
                child.parent = self
        except TypeError:
            self._payload = expr

    ### SPECIAL METHODS ###

    def __contains__(self, expr):
        r'''True when payload tree contains `expr`.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[-1] in tree
            True

        Otherwise false:

            >>> tree[-1][-1] in tree
            False

        Returns boolean.
        '''
        return expr in self._children

    def __eq__(self, expr):
        r'''True when `expr` is the same type as tree
        and when the payload of all subtrees are equal.

        ::

            >>> sequence_1 = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree_1 = datastructuretools.PayloadTree(sequence_1)
            >>> sequence_2 = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree_2 = datastructuretools.PayloadTree(sequence_2)
            >>> sequence_3 = [[0, 1], [2, 3], [4, 5]]
            >>> tree_3 = datastructuretools.PayloadTree(sequence_3)

        ::

            >>> tree_1 == tree_1
            True
            >>> tree_1 == tree_2
            True
            >>> tree_1 == tree_3
            False
            >>> tree_2 == tree_1
            True
            >>> tree_2 == tree_2
            True
            >>> tree_2 == tree_3
            False
            >>> tree_3 == tree_1
            False
            >>> tree_3 == tree_2
            False
            >>> tree_3 == tree_3
            True

        Returns boolean.
        '''
        if isinstance(expr, type(self)):
            if self.payload is not None or expr.payload is not None:
                return self.payload == expr.payload
            if len(self) == len(expr):
                for x, y in zip(
                    self._noncyclic_children, expr._noncyclic_children):
                    if not x == y:
                        return False
                else:
                    return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats payload tree.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> print format(tree)
            datastructuretools.PayloadTree(
                [
                    [0, 1],
                    [2, 3],
                    [4, 5],
                    [6, 7],
                    ]
                )

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatManager.get_storage_format(self)
        return str(self)

    def __getitem__(self, expr):
        r'''Gets `expr` from payload tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[-1]
            PayloadTree([6, 7])

        Gets slice from payload tree:

        ::

            >>> tree[-2:]
            [PayloadTree([4, 5]), PayloadTree([6, 7])]

        Returns node.
        '''
        return self._children[expr]

    def __getstate__(self):
        r'''Gets object state.
        '''
        return vars(self)

    def __len__(self):
        r'''Number of children in payload tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> len(tree)
            4

        Returns nonnegative integer.
        '''
        return len(self._children)

    def __repr__(self):
        r'''Gets interpreter representation of payload tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree
            PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])

        Returns string.
        '''
        return '{}({!s})'.format(type(self).__name__, self)

    def __str__(self):
        r'''String representation of payload tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> str(tree)
            '[[0, 1], [2, 3], [4, 5], [6, 7]]'

        Returns string.
        '''
        if self.payload is None:
            return '[{}]'.format(
                ', '.join([str(x) for x in self._noncyclic_children])
                )
        else:
            return repr(self.payload)

    ### PRIVATE PROPERTIES ###

    @property
    def _input_argument(self):
        return eval(str(self))

    @property
    def _noncyclic_children(self):
        return list(self._children)

    @property
    def _storage_format_specification(self):
        from abjad.tools import systemtools
        positional_argument_values = (
            self._input_argument,
            )
        return systemtools.StorageFormatSpecification(
            self,
            keyword_argument_names=(),
            positional_argument_values=positional_argument_values,
            )

    ### PRIVATE METHODS ###

    def _initialize_and_wrap_nodes_with_type(self, expr, type_):
        self._children = self._initialize_children_list()
        self.parent = None
        try:
            self.payload = None
            for element in expr:
                child = type_(element)
                self._children.append(child)
                child.parent = self
        except TypeError:
            self.payload = expr

    ### PUBLIC PROPERTIES ###

    @property
    def children(self):
        r'''Children of payload tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[1].children
            (PayloadTree(2), PayloadTree(3))

        Returns tuple of zero or more nodes.
        '''
        return tuple(self._children)

    @property
    def depth(self):
        r'''Depth of payload tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[1].depth
            2

        Returns nonnegative integer.
        '''
        levels = set([])
        for node in self.iterate_depth_first():
            levels.add(node.level)
        return max(levels) - self.level + 1

    @property
    def graphviz_format(self):
        r'''Graphviz format of payload tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)
            >>> print tree.graphviz_format
            digraph G {
                node_0 [label="",
                    shape=circle];
                node_1 [label="",
                    shape=circle];
                node_2 [label=0,
                    shape=box];
                node_3 [label=1,
                    shape=box];
                node_4 [label="",
                    shape=circle];
                node_5 [label=2,
                    shape=box];
                node_6 [label=3,
                    shape=box];
                node_7 [label="",
                    shape=circle];
                node_8 [label=4,
                    shape=box];
                node_9 [label=5,
                    shape=box];
                node_10 [label="",
                    shape=circle];
                node_11 [label=6,
                    shape=box];
                node_12 [label=7,
                    shape=box];
                node_0 -> node_1;
                node_0 -> node_10;
                node_0 -> node_4;
                node_0 -> node_7;
                node_1 -> node_2;
                node_1 -> node_3;
                node_10 -> node_11;
                node_10 -> node_12;
                node_4 -> node_5;
                node_4 -> node_6;
                node_7 -> node_8;
                node_7 -> node_9;
            }
            
        Returns string.
        '''
        return self.graphviz_graph.graphviz_format

    @property
    def graphviz_graph(self):
        r'''The GraphvizGraph representation of payload tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> graph = tree.graphviz_graph
            >>> topleveltools.graph(graph) # doctest: +SKIP

        Returns graphviz graph.
        '''
        from abjad.tools import documentationtools
        graph = documentationtools.GraphvizGraph(name='G')
        node_mapping = {}
        for node in self.iterate_depth_first():
            graphviz_node = documentationtools.GraphvizNode()
            if node.children:
                graphviz_node.attributes['shape'] = 'circle'
                graphviz_node.attributes['label'] = '""'
            else:
                graphviz_node.attributes['shape'] = 'box'
                graphviz_node.attributes['label'] = str(node.payload)
            graph.append(graphviz_node)
            node_mapping[node] = graphviz_node
            if node.parent is not None:
                documentationtools.GraphvizEdge()(
                    node_mapping[node.parent],
                    node_mapping[node],
                    )
        return graph

    @property
    def improper_parentage(self):
        r'''Improper parentage of payload tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[1].improper_parentage
            (PayloadTree([2, 3]), PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]]))

        Returns tuple of one or more nodes.
        '''
        result = [self]
        current = self.parent
        while current is not None:
            result.append(current)
            current = current.parent
        return tuple(result)

    @property
    def index_in_parent(self):
        r'''Index of node in parent of node.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[1].index_in_parent
            1

        Returns nonnegative integer.
        '''
        if self.parent is not None:
            return self.parent.index(self)
        else:
            return None

    @property
    def level(self):
        r'''Level of node.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[1].level
            1

        Returns nonnegative integer.
        '''
        return len(self.proper_parentage)

    @property
    def manifest_payload(self):
        r'''Manifest payload of payload tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree.manifest_payload
            [0, 1, 2, 3, 4, 5, 6, 7]

        ::

            >>> tree[-1].manifest_payload
            [6, 7]

        ::

            >>> tree[-1][-1].manifest_payload
            [7]

        Returns list.
        '''
        if 0 <= self.level:
            return [x.payload for x in self.iterate_at_level(-1)]
        else:
            return [self.payload]

    @property
    def negative_level(self):
        r'''Negative level of node.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[1].negative_level
            -2

        Returns negative integer.
        '''
        return self.level - self.root.depth

    @property
    def payload(self):
        r'''Payload of node.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        Returns none for interior node:

        ::

            >>> tree.payload is None
            True

        ::

            >>> tree[-1].payload is None
            True

        Returns unwrapped payload for leaf node:

            >>> tree[-1][-1].payload
            7

        Returns arbitrary expression or none.
        '''
        return self._payload

    @property
    def position(self):
        r'''Position of node relative to root.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[1].position
            (1,)

        Returns tuple of zero or more nonnegative integers.
        '''
        result = []
        for node in self.improper_parentage:
            if node.parent is not None:
                result.append(node.index_in_parent)
        result.reverse()
        return tuple(result)

    @property
    def proper_parentage(self):
        r'''Proper parentage of node.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[1].proper_parentage
            (PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]]),)


        Returns tuple of zero or more nodes.
        '''
        return self.improper_parentage[1:]

    @property
    def root(self):
        r'''Root of payload tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[1].proper_parentage
            (PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]]),)

        Returns node.
        '''
        return self.improper_parentage[-1]

    @property
    def width(self):
        r'''Number of leaves in payload tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[1].width
            2

        Returns nonnegative integer.
        '''
        return len(list(self.iterate_at_level(-1)))

    ### PRIVATE METHODS ###

    def _get_next_n_nodes_at_level_helper(self, 
        n, level, nodes_must_be_complete=False):
        result = []
        self_is_found = False
        first_node_returned_is_trimmed = False
        if n < 0:
            reverse = True
        else:
            reverse = False
        n = abs(n)
        if hasattr(self.root, 'iterate_forever_depth_first'):
            generator = self.root.iterate_forever_depth_first(reverse=reverse)
        else:
            generator = self.root.iterate_depth_first(reverse=reverse)
        for node in generator:
            if len(result) == n:
                if not first_node_returned_is_trimmed or \
                    not nodes_must_be_complete:
                    return result
            if len(result) == n + 1:
                return result
            if node is self:
                self_is_found = True
                # test whether node to return is higher in tree than self;
                # or-clause allows for test of either nonnegative 
                # or negative level
                if ((0 <= level) and level < self.level) or \
                   ((level < 0) and level < self.negative_level):
                    first_node_returned_is_trimmed = True
                    subtree_to_trim = node.parent
                    # find subtree to trim where level is nonnegative
                    if 0 <= level:
                        while level < subtree_to_trim.level:
                            subtree_to_trim = subtree_to_trim.parent
                    # find subtree to trim where level is negative
                    else:
                        while subtree_to_trim.negative_level < level:
                            subtree_to_trim = subtree_to_trim.parent
                    position_of_descendant = \
                        subtree_to_trim.get_position_of_descendant(node)
                    first_subtree = copy.deepcopy(subtree_to_trim)
                    reference_node = \
                        first_subtree.get_node_at_position(
                            position_of_descendant)
                    reference_node.remove_to_root(reverse=reverse)
                    result.append(first_subtree)
            if self_is_found:
                if node is not self:
                    if node.is_at_level(level):
                        result.append(node)
        else:
            message = 'not enough nodes at level {}.'.format(level)
            raise ValueError(message)

    def _initialize_children_list(self):
        return []

    ### PUBLIC METHODS ###

    def get_manifest_payload_of_next_n_nodes_at_level(self, n, level):
        r'''Gets manifest payload of next `n` nodes at `level` from node.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        Gets manifest paylaod of next 4 nodes at level 2:

        ::

            >>> tree[0][0].get_manifest_payload_of_next_n_nodes_at_level(4, 2)
            [1, 2, 3, 4]

        Gets manifest paylaod of next 3 nodes at level 1:

        ::

            >>> tree[0][0].get_manifest_payload_of_next_n_nodes_at_level(3, 1)
            [1, 2, 3, 4, 5]

        Gets manifest paylaod of next node at level 0:

        ::

            >>> tree[0][0].get_manifest_payload_of_next_n_nodes_at_level(1, 0)
            [1, 2, 3, 4, 5, 6, 7]

        Gets manifest paylaod of next 4 nodes at level -1:

        ::

            >>> tree[0][0].get_manifest_payload_of_next_n_nodes_at_level(4, -1)
            [1, 2, 3, 4]

        Gets manifest paylaod of next 3 nodes at level -2:

        ::

            >>> tree[0][0].get_manifest_payload_of_next_n_nodes_at_level(3, -2)
            [1, 2, 3, 4, 5]

        Gets manifest paylaod of previous 4 nodes at level 2:

        ::

            >>> tree[-1][-1].get_manifest_payload_of_next_n_nodes_at_level(-4, 2)
            [6, 5, 4, 3]

        Gets manifest paylaod of previous 3 nodes at level 1:

        ::

            >>> tree[-1][-1].get_manifest_payload_of_next_n_nodes_at_level(-3, 1)
            [6, 5, 4, 3, 2]

        Gets manifest paylaod of previous node at level 0:

        ::

            >>> tree[-1][-1].get_manifest_payload_of_next_n_nodes_at_level(-1, 0)
            [6, 5, 4, 3, 2, 1, 0]

        Gets manifest paylaod of previous 4 nodes at level -1:

        ::

            >>> tree[-1][-1].get_manifest_payload_of_next_n_nodes_at_level(-4, -1)
            [6, 5, 4, 3]

        Gets manifest paylaod of previous 3 nodes at level -2:

        ::

            >>> tree[-1][-1].get_manifest_payload_of_next_n_nodes_at_level(-3, -2)
            [6, 5, 4, 3, 2]

        Trims first node if necessary.

        Returns list of arbitrary values.
        '''
        result = []
        nodes = self.get_next_n_nodes_at_level(n, level)
        for node in nodes:
            if 0 <= n:
                result.extend(node.manifest_payload)
            else:
                result.extend(reversed(node.manifest_payload))
        return result

    def get_next_n_complete_nodes_at_level(self, n, level):
        r'''Gets next `n` complete nodes at `level` from node.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        Gets next 4 nodes at level 2:

        ::

            >>> tree[0][0].get_next_n_complete_nodes_at_level(4, 2)
            [PayloadTree(1), PayloadTree(2), PayloadTree(3), PayloadTree(4)]

        Gets next 3 nodes at level 1:

        ::

            >>> tree[0][0].get_next_n_complete_nodes_at_level(3, 1)
            [PayloadTree([1]), PayloadTree([2, 3]), PayloadTree([4, 5]), PayloadTree([6, 7])]

        Gets next 4 nodes at level -1:

        ::

            >>> tree[0][0].get_next_n_complete_nodes_at_level(4, -1)
            [PayloadTree(1), PayloadTree(2), PayloadTree(3), PayloadTree(4)]

        Gets next 3 nodes at level -2:

        ::

            >>> tree[0][0].get_next_n_complete_nodes_at_level(3, -2)
            [PayloadTree([1]), PayloadTree([2, 3]), PayloadTree([4, 5]), PayloadTree([6, 7])]

        Gets previous 4 nodes at level 2:

        ::

            >>> tree[-1][-1].get_next_n_complete_nodes_at_level(-4, 2)
            [PayloadTree(6), PayloadTree(5), PayloadTree(4), PayloadTree(3)]

        Gets previous 3 nodes at level 1:

        ::

            >>> tree[-1][-1].get_next_n_complete_nodes_at_level(-3, 1)
            [PayloadTree([6]), PayloadTree([4, 5]), PayloadTree([2, 3]), PayloadTree([0, 1])]


        Gets previous 4 nodes at level -1:

        ::

            >>> tree[-1][-1].get_next_n_complete_nodes_at_level(-4, -1)
            [PayloadTree(6), PayloadTree(5), PayloadTree(4), PayloadTree(3)]

        Gets previous 3 nodes at level -2:

        ::

            >>> tree[-1][-1].get_next_n_complete_nodes_at_level(-3, -2)
            [PayloadTree([6]), PayloadTree([4, 5]), PayloadTree([2, 3]), PayloadTree([0, 1])]

        Trims first node if necessary.

        Returns list of nodes.
        '''
        return self._get_next_n_nodes_at_level_helper(
            n, level, nodes_must_be_complete=True)

    def get_next_n_nodes_at_level(self, n, level):
        r'''Gets next `n` nodes at `level` from node.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        Gets next 4 nodes at level 2:

        ::

            >>> tree[0][0].get_next_n_nodes_at_level(4, 2)
            [PayloadTree(1), PayloadTree(2), PayloadTree(3), PayloadTree(4)]

        Gets next 3 nodes at level 1:

        ::

            >>> tree[0][0].get_next_n_nodes_at_level(3, 1)
            [PayloadTree([1]), PayloadTree([2, 3]), PayloadTree([4, 5])]

        Gets next node at level 0:

        ::

            >>> tree[0][0].get_next_n_nodes_at_level(1, 0)
            [PayloadTree([[1], [2, 3], [4, 5], [6, 7]])]

        Gets next 4 nodes at level -1:

        ::

            >>> tree[0][0].get_next_n_nodes_at_level(4, -1)
            [PayloadTree(1), PayloadTree(2), PayloadTree(3), PayloadTree(4)]

        Gets next 3 nodes at level -2:

        ::

            >>> tree[0][0].get_next_n_nodes_at_level(3, -2)
            [PayloadTree([1]), PayloadTree([2, 3]), PayloadTree([4, 5])]

        Gets previous 4 nodes at level 2:

        ::

            >>> tree[-1][-1].get_next_n_nodes_at_level(-4, 2)
            [PayloadTree(6), PayloadTree(5), PayloadTree(4), PayloadTree(3)]

        Gets previous 3 nodes at level 1:

        ::

            >>> tree[-1][-1].get_next_n_nodes_at_level(-3, 1)
            [PayloadTree([6]), PayloadTree([4, 5]), PayloadTree([2, 3])]

        Gets previous node at level 0:

        ::

            >>> tree[-1][-1].get_next_n_nodes_at_level(-1, 0)
            [PayloadTree([[0, 1], [2, 3], [4, 5], [6]])]

        Gets previous 4 nodes at level -1:

        ::

            >>> tree[-1][-1].get_next_n_nodes_at_level(-4, -1)
            [PayloadTree(6), PayloadTree(5), PayloadTree(4), PayloadTree(3)]

        Gets previous 3 nodes at level -2:

        ::

            >>> tree[-1][-1].get_next_n_nodes_at_level(-3, -2)
            [PayloadTree([6]), PayloadTree([4, 5]), PayloadTree([2, 3])]

        Trims first node if necessary.

        Returns list of nodes.
        '''
        return self._get_next_n_nodes_at_level_helper(
            n, level, nodes_must_be_complete=False)

    def get_node_at_position(self, position):
        r'''Gets node at `position`.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree.get_node_at_position((2, 1))
            PayloadTree(5)

        Returns node.
        '''
        result = self
        for idx in position:
            result = result[idx]
        return result

    def get_position_of_descendant(self, descendant):
        r'''Gets position of `descendent` relative to node 
        rather than relative to root.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[3].get_position_of_descendant(tree[3][0])
            (0,)

        Returns tuple of zero or more nonnegative integers.
        '''
        if descendant is self:
            return ()
        else:
            return descendant.position[len(self.position):]

    def index(self, node):
        r'''Index of `node`.

        ::

            >>> sequence = [0, 1, 2, 2, 3, 4]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> for node in tree:
            ...     node, tree.index(node)
            (PayloadTree(0), 0)
            (PayloadTree(1), 1)
            (PayloadTree(2), 2)
            (PayloadTree(2), 3)
            (PayloadTree(3), 4)
            (PayloadTree(4), 5)

        Returns nonnegative integer.
        '''
        for i, current_node in enumerate(self):
            if current_node is node:
                return i
        message = '{!r} is not in tree.'.format(node)
        raise ValueError(message)

    def is_at_level(self, level):
        r'''True when node is at `level` in containing tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree[1][1].is_at_level(-1)
            True

        Otherwise false:

        ::

            >>> tree[1][1].is_at_level(0)
            False

        Works for positive, negative and zero-valued `level`.

        Returns boolean.
        '''
        if (0 <= level and self.level == level) or \
            self.negative_level == level:
            return True
        else:
            return False

    def iterate_at_level(self, level, reverse=False):
        r'''Iterates tree at `level`.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        Left-to-right examples:

        ::

            >>> for x in tree.iterate_at_level(0): x
            ...
            PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])

        ::

            >>> for x in tree.iterate_at_level(1): x
            ...
            PayloadTree([0, 1])
            PayloadTree([2, 3])
            PayloadTree([4, 5])
            PayloadTree([6, 7])

        ::

            >>> for x in tree.iterate_at_level(2): x
            ...
            PayloadTree(0)
            PayloadTree(1)
            PayloadTree(2)
            PayloadTree(3)
            PayloadTree(4)
            PayloadTree(5)
            PayloadTree(6)
            PayloadTree(7)

        ::

            >>> for x in tree.iterate_at_level(-1): x
            ...
            PayloadTree(0)
            PayloadTree(1)
            PayloadTree(2)
            PayloadTree(3)
            PayloadTree(4)
            PayloadTree(5)
            PayloadTree(6)
            PayloadTree(7)

        ::

            >>> for x in tree.iterate_at_level(-2): x
            ...
            PayloadTree([0, 1])
            PayloadTree([2, 3])
            PayloadTree([4, 5])
            PayloadTree([6, 7])

        ::

            >>> for x in tree.iterate_at_level(-3): x
            ...
            PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])

        Right-to-left examples:

        ::

            >>> for x in tree.iterate_at_level(0, reverse=True): x
            ...
            PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])

        ::

            >>> for x in tree.iterate_at_level(1, reverse=True): x
            ...
            PayloadTree([6, 7])
            PayloadTree([4, 5])
            PayloadTree([2, 3])
            PayloadTree([0, 1])

        ::

            >>> for x in tree.iterate_at_level(2, reverse=True): x
            ...
            PayloadTree(7)
            PayloadTree(6)
            PayloadTree(5)
            PayloadTree(4)
            PayloadTree(3)
            PayloadTree(2)
            PayloadTree(1)
            PayloadTree(0)

        ::

            >>> for x in tree.iterate_at_level(-1, reverse=True): x
            ...
            PayloadTree(7)
            PayloadTree(6)
            PayloadTree(5)
            PayloadTree(4)
            PayloadTree(3)
            PayloadTree(2)
            PayloadTree(1)
            PayloadTree(0)

        ::

            >>> for x in tree.iterate_at_level(-2, reverse=True): x
            ...
            PayloadTree([6, 7])
            PayloadTree([4, 5])
            PayloadTree([2, 3])
            PayloadTree([0, 1])

        ::

            >>> for x in tree.iterate_at_level(-3, reverse=True): x
            ...
            PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])

        Returns node generator.
        '''
        for x in self.iterate_depth_first(reverse=reverse):
            if 0 <= level:
                if x.level == level:
                    yield x
            else:
                if x.negative_level == level:
                    yield x

    def iterate_depth_first(self, reverse=False):
        r'''Iterates tree depth-first.

        ..  container:: example

            **Example 1.** Iterate tree depth-first from left to right:

            ::

                >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
                >>> tree = datastructuretools.PayloadTree(sequence)

            ::

                >>> for node in tree.iterate_depth_first(): node
                ...
                PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])
                PayloadTree([0, 1])
                PayloadTree(0)
                PayloadTree(1)
                PayloadTree([2, 3])
                PayloadTree(2)
                PayloadTree(3)
                PayloadTree([4, 5])
                PayloadTree(4)
                PayloadTree(5)
                PayloadTree([6, 7])
                PayloadTree(6)
                PayloadTree(7)

        ..  container::

            **Example 2.** Iterate tree depth-first from right to left:

            ::

                >>> for node in tree.iterate_depth_first(reverse=True): node
                ...
                PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])
                PayloadTree([6, 7])
                PayloadTree(7)
                PayloadTree(6)
                PayloadTree([4, 5])
                PayloadTree(5)
                PayloadTree(4)
                PayloadTree([2, 3])
                PayloadTree(3)
                PayloadTree(2)
                PayloadTree([0, 1])
                PayloadTree(1)
                PayloadTree(0)

        Returns node generator.
        '''
        yield self
        iterable_self = self
        if reverse:
            iterable_self = reversed(self)
        for x in iterable_self:
            for y in x.iterate_depth_first(reverse=reverse):
                yield y

    def iterate_payload(self, reverse=False):
        r'''Iterates payload of tree.

        ..  container:: example

            **Example 1.** Iterates payload from left to right:

            ::

                >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
                >>> tree = datastructuretools.PayloadTree(sequence)

            ::

                >>> for element in tree.iterate_payload():
                ...     element
                ...
                0
                1
                2
                3
                4
                5
                6
                7

        ..  container:: example

            **Example 2.** Iterates payload from right to left:

            ::

                >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
                >>> tree = datastructuretools.PayloadTree(sequence)

            ::

                >>> for element in tree.iterate_payload(reverse=True):
                ...     element
                ...
                7
                6
                5
                4
                3
                2
                1
                0

        Returns payload generator.
        '''
        for leaf_node in self.iterate_at_level(-1, reverse=reverse):
            yield leaf_node.payload

    def remove_node(self, node):
        r'''Removes `node` from tree.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree.remove_node(tree[1])

        ::

            >>> tree
            PayloadTree([[0, 1], [4, 5], [6, 7]])

        Returns none.
        '''
        node.parent._children.remove(node)
        node.parent = None

    def remove_to_root(self, reverse=False):
        r'''Removes node and all nodes left of node to root.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]

        ::

            >>> tree = datastructuretools.PayloadTree(sequence)
            >>> tree[0][0].remove_to_root()
            >>> tree
            PayloadTree([[1], [2, 3], [4, 5], [6, 7]])

        ::

            >>> tree = datastructuretools.PayloadTree(sequence)
            >>> tree[0][1].remove_to_root()
            >>> tree
            PayloadTree([[2, 3], [4, 5], [6, 7]])

        ::

            >>> tree = datastructuretools.PayloadTree(sequence)
            >>> tree[1].remove_to_root()
            >>> tree
            PayloadTree([[4, 5], [6, 7]])

        Modifies in-place to root.

        Returns none.
        '''
        ## trim left-siblings of self and self
        parent = self.parent
        if reverse:
            iterable_parent = reversed(parent)
        else:
            iterable_parent = parent[:]
        for sibling in iterable_parent:
            sibling.parent.remove_node(sibling)
            ## break and do not remove siblings to right of self
            if sibling is self:
                break
        ## trim parentage
        for node in parent.improper_parentage:
            if node.parent is not None:
                iterable_parent = node.parent[:]
                if reverse:
                    iterable_parent = reversed(node.parent)
                else:
                    iterable_parent = node.parent[:]
                for sibling in iterable_parent:
                    if sibling is node:
                        # remove node now if it was emptied earlier
                        if not len(sibling):
                            sibling.parent.remove_node(sibling)
                        break
                    else:
                        sibling.parent.remove_node(sibling)

    def to_nested_lists(self):
        r'''Changes tree to nested lists.

        ::

            >>> sequence = [[0, 1], [2, 3], [4, 5], [6, 7]]
            >>> tree = datastructuretools.PayloadTree(sequence)

        ::

            >>> tree
            PayloadTree([[0, 1], [2, 3], [4, 5], [6, 7]])

        ::

            >>> tree.to_nested_lists()
            [[0, 1], [2, 3], [4, 5], [6, 7]]

        Returns list of lists.
        '''
        if self.payload is not None:
            message = 'leaf node is not iterable.'
            raise TypeError(message)
        else:
            result = []
            for child in self._noncyclic_children:
                if child.payload is not None:
                    result.append(child.payload)
                else:
                    result.append(child.to_nested_lists())
            return result



# -*- encoding: utf-8 -*-
from abjad import *


def test_sequencetools_split_sequence_extended_to_weights_01():

    parts = sequencetools.split_sequence_extended_to_weights([1, 2, 3, 4, 5], [7, 7, 7], overhang=True)
    assert parts == [[1, 2, 3, 1], [3, 4], [1, 1, 2, 3], [4, 5]]


def test_sequencetools_split_sequence_extended_to_weights_02():

    parts = sequencetools.split_sequence_extended_to_weights([1, 2, 3, 4, 5], [7, 7, 7], overhang=False)
    assert parts == [[1, 2, 3, 1], [3, 4], [1, 1, 2, 3]]



# -*- encoding: utf-8 -*-


def remap_sequence_by_range_pairs(sequence, range_pairs):
    r'''Remaps `sequence` by `range_pairs`:

    ::

        >>> sequence = [9, 14, 4, 1, 7, 5, 9, 6, 2, 10, 15, 20, 8, 4, 0, 7]
        >>> range_pairs = [
        ...     ((2, 10), (3, 5)),
        ...     ((10, 20), (6, 8)),
        ...     ]
        >>> sequencetools.remap_sequence_by_range_pairs(sequence, range_pairs)
        [4, 7, 5, 1, 5, 3, 4, 4, 3, 7, 8, 7, 3, 5, 0, 5]

    Returns newly created `sequence` object.
    '''
    result = []
    for old_element in sequence:
        new_element = old_element
        for input_range, output_range in range_pairs:
            input_low, input_high = input_range
            if input_low <= old_element <= input_high:
                output_low, output_high = output_range
                output_difference = output_high - output_low
                new_element -= input_low
                new_element %= (output_difference + 1)
                new_element += output_low
        result.append(new_element)
    return type(sequence)(result)



# -*- encoding: utf-8 -*-
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__misc__comments_01():
    r'''Comments are ignored.
    '''

    target = Staff()
    input = r'''\new Staff { %{ HOO HAH %} }'''
    parser = LilyPondParser()
    result = parser(input)
    assert format(target) == format(result) and target is not result



# -*- encoding: utf-8 -*-


def is_integer_singleton(expr):
    r'''True when `expr` is an integer tuple of of length ``1``:

    ::

        >>> sequencetools.is_integer_singleton((19,))
        True

    Otherwise false:

    ::

        >>> sequencetools.is_integer_singleton(('text',))
        False

    Returns boolean.
    '''

    return isinstance(expr, tuple) and len(expr) == 1 and isinstance(expr[0], int)



def override(expr):
    r'''Overrides `expr`.

    Returns LilyPond grob manager.
    '''
    from abjad.tools import lilypondnametools

    if getattr(expr, '_override', None) is None:
        plug_in = lilypondnametools.LilyPondGrobNameManager()
        expr._override = plug_in

    return expr._override



# -*- encoding: utf-8 -*-


def get_sequence_elements_frequency_distribution(sequence):
    '''Get `sequence` elements frequency distribution:

    ::

        >>> sequence = [1, 3, 3, 3, 2, 1, 1, 2, 3, 3, 1, 2]

    ::

        >>> sequencetools.get_sequence_elements_frequency_distribution(sequence)
        [(1, 4), (2, 3), (3, 5)]

    Returns list of element / count pairs.
    '''
    from abjad.tools import sequencetools

    result = sorted(sequence)
    result = sequencetools.partition_sequence_by_value_of_elements(result)
    result = [(x[0], len(x)) for x in result]
    return result



# -*- encoding: utf-8 -*-
import argparse
import os
import shutil
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript


class CleanScript(DirectoryScript):
    r'''Removes *.pyc, *.swp files and __pycache__ and tmp* directories 
    recursively in a path.

    ..  shell::

        ajv clean --help

    '''

    ### PUBLIC PROPERTIES ###

    @property
    def alias(self):
        r'''Alias of script.

        Returns ``'clean'``.
        '''
        return 'clean'

    @property
    def long_description(self):
        r'''Long description of scrip.

        Returns string or none.
        '''
        return None

    @property
    def scripting_group(self):
        r'''Scripting groupt of script.

        Returns none.
        '''
        return None

    @property
    def short_description(self):
        r'''Short description of script.

        Returns string.
        '''
        return 'Clean *.pyc, *.swp, __pycache__ and tmp* files' + \
            ' and folders from PATH.'

    @property
    def version(self):
        r'''Version of script.

        Returns float.
        '''
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        r'''Processes `args`.

        Returns none.
        '''
        if not args.pyc and not args.pycache and \
            not args.swp and not args.tmp:
            args.pyc, args.pycache, args.swp, args.tmp = True, True, True, True

        print 'Cleaning...'
        if args.pyc:
            print '\t*.pyc files'
        if args.swp:
            print '\t*.swp files'
        if args.pycache:
            print '\t__pycache__ directories'
        if args.tmp:
            print '\ttmp* directories'

        for root, dirs, files in os.walk(args.path):
            if '.svn' in dirs:
                dirs.remove('.svn')

            extensions = ()
            if args.pyc:
                extensions += ('.pyc',)
            if args.swp:
                extensions += ('.swp',)
            for file in files:
                if file.endswith(extensions):
                    os.remove(os.path.join(root, file))

            directories_to_remove = []
            predicate = lambda x: False
            if args.pycache and args.tmp:
                predicate = lambda x: x == '__pycache__' or x.startswith('tmp')
            elif args.pycache:
                predicate = lambda x: x == '__pycache__'
            elif args.tmp:
                predicate = lambda x: x.startswith('tmp')
            for dir in dirs:
                if predicate(dir):
                    shutil.rmtree(os.path.join(root, dir))
                    directories_to_remove.append(dir)
            for dir in directories_to_remove:
                dirs.remove(dir)

    def setup_argument_parser(self, parser):
        r'''Sets up argument `parser`.

        Returns none.
        '''
        parser.add_argument('path',
            default=os.getcwd(),
            help='directory tree to be recursed over',
            nargs='?',
            type=self._validate_path,
            )
        parser.add_argument('--pyc',
            action='store_true',
            help='delete *.pyc files',
            )
        parser.add_argument('--pycache',
            action='store_true',
            help='delete __pycache__ folders',
            )
        parser.add_argument('--swp',
            action='store_true',
            help='delete Vim *.swp file',
            )
        parser.add_argument('--tmp',
            action='store_true',
            help='delete tmp* folders',
            )



# -*- encoding: utf-8 -*-
from abjad.tools.spannertools.Spanner import Spanner


class PianoPedalSpanner(Spanner):
    r'''A piano pedal spanner.

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> pedal = spannertools.PianoPedalSpanner()
        >>> attach(pedal, staff[:])
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> print format(staff)
        \new Staff {
            \set Staff.pedalSustainStyle = #'mixed
            c'8 \sustainOn
            d'8
            e'8
            f'8 \sustainOff
        }

    '''

    ### CLASS VARIABLES ###

    _kinds = {
        'sustain': (r'\sustainOn', r'\sustainOff'),
        'sostenuto':(r'\sostenutoOn', r'\sostenutoOff'),
        'corda': (r'\unaCorda', r'\treCorde'),
        }

    _styles = [
        'text',
        'bracket',
        'mixed',
        ]

    ### INITIALIZER ###

    def __init__(
        self, 
        components=None,
        overrides=None,
        ):
        Spanner.__init__(
            self, 
            components,
            overrides=overrides,
            )
        self.kind = 'sustain'
        self.style = 'mixed'

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new.kind = self.kind
        new.style = self.style

    def _format_before_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r"\set Staff.pedalSustainStyle = #'%s" % self.style)
        return result

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(self._kinds[self.kind][0])
        if self._is_my_last_leaf(leaf):
            result.append(self._kinds[self.kind][1])
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def kind(self):
        r'''Get piano pedal spanner kind:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.PianoPedalSpanner()
            >>> attach(spanner, staff[:])
            >>> spanner.kind
            'sustain'

        Set piano pedal spanner kind:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.PianoPedalSpanner()
            >>> attach(spanner, staff[:])
            >>> spanner.kind = 'sostenuto'
            >>> spanner.kind
            'sostenuto'

        Acceptable values ``'sustain'``, ``'sostenuto'``, ``'corda'``.
        '''
        return self._kind

    @kind.setter
    def kind(self, arg):
        if not arg in self._kinds.keys():
            raise ValueError("Type must be in %s" % self._kinds.keys())
        self._kind = arg

    @property
    def style(self):
        r'''Get piano pedal spanner style:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.PianoPedalSpanner()
            >>> attach(spanner, staff[:])
            >>> spanner.style
            'mixed'

        Set piano pedal spanner style:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.PianoPedalSpanner()
            >>> attach(spanner, staff[:])
            >>> spanner.style = 'bracket'
            >>> spanner.style
            'bracket'

        Acceptable values ``'mixed'``, ``'bracket'``, ``'text'``.
        '''
        return self._style

    @style.setter
    def style(self, arg):
        if not arg in self._styles:
            raise ValueError("Style must be in %s" % self._styles)
        self._style = arg



# -*- encoding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_MIDIBlock_01():

    MIDI_block = lilypondfiletools.MIDIBlock()

    assert systemtools.TestManager.compare(
        MIDI_block,
        r'''
        \midi {}
        '''
        )



# -*- encoding: utf-8 -*-
from abjad.tools import sequencetools


def numbered_inversion_equivalent_interval_class_dictionary(pitches):
    r'''Change named `pitches` to numbered inversion-equivalent interval-class
    number dictionary:

    ::

        >>> chord = Chord("<c' d' b''>4")
        >>> vector = pitchtools.numbered_inversion_equivalent_interval_class_dictionary(
        ... chord.written_pitches)
        >>> for i in range(7):
        ...     print '\t%s\t%s' % (i, vector[i])
        ...
            0  0
            1  1
            2  1
            3  1
            4  0
            5  0
            6  0

    Returns dictionary.
    '''
    from abjad.tools import pitchtools

    numbers = []

    for pitch in pitches:
        if not isinstance(pitch, pitchtools.NamedPitch):
            raise ValueError
        numbers.append(pitch.pitch_number)

    pairs = sequencetools.yield_all_unordered_pairs_of_sequence(numbers)

    intervals = []

    for pair in pairs:
        interval = max(pair) - min(pair)
        interval %= 12
        if 6 < interval:
            interval = 12 - interval
        intervals.append(interval)

    vector = {}

    # 12-ET pitches only
    if all(isinstance(interval, int) for interval in intervals):
        for i in range(7):
            vector[i] = intervals.count(i)
    # 24-ET pitches included
    else:
        for i in range(13):
            if i % 2 == 0:
                key = i / 2
            else:
                key = i / 2.0
            vector[key] = intervals.count(key)

    return vector



# -*- encoding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools.topleveltools import override


def color_measure(measure, color='red'):
    r'''Color `measure` with `color`:

    ::

        >>> measure = Measure((2, 8), "c'8 d'8")

    ..  doctest::

        >>> print format(measure)
        {
            \time 2/8
            c'8
            d'8
        }

    ::

        >>> show(measure) # doctest: +SKIP

    ::

        >>> labeltools.color_measure(measure, 'red')
        Measure((2, 8), "c'8 d'8")

    ..  doctest::

        >>> print format(measure)
        {
            \override Beam #'color = #red
            \override Dots #'color = #red
            \override NoteHead #'color = #red
            \override Staff.TimeSignature #'color = #red
            \override Stem #'color = #red
            \time 2/8
            c'8
            d'8
            \revert Beam #'color
            \revert Dots #'color
            \revert NoteHead #'color
            \revert Staff.TimeSignature #'color
            \revert Stem #'color
        }

    ::

        >>> show(measure) # doctest: +SKIP

    Returns colored `measure`.

    Color names appear in LilyPond Learning Manual appendix B.5.
    '''

    # check measure type
    if not isinstance(measure, scoretools.Measure):
        message = 'must be measure: {}'.format(measure)
        raise TypeError(message)

    # color measure
    override(measure).beam.color = color
    override(measure).dots.color = color
    override(measure).staff.time_signature.color = color
    override(measure).note_head.color = color
    override(measure).stem.color = color

    # return measure
    return measure



# -*- encoding: utf-8 -*-


def transpose_named_pitch_by_numbered_interval_and_respell(
    pitch, staff_spaces, numbered_interval):
    '''Transpose named pitch by `numbered_interval` and respell `staff_spaces`
    above or below:

    ::

        >>> pitch = NamedPitch(0)

    ::

        >>> pitchtools.transpose_named_pitch_by_numbered_interval_and_respell(
        ...     pitch, 1, 0.5)
        NamedPitch("dtqf'")

    Returns new named pitch.
    '''
    from abjad.tools import pitchtools

    pitch_number = pitch.pitch_number + numbered_interval
    diatonic_pitch_class_number = (pitch.diatonic_pitch_class_number + staff_spaces) % 7
    diatonic_pitch_class_name = \
        pitchtools.PitchClass._diatonic_pitch_class_number_to_diatonic_pitch_class_name[
            diatonic_pitch_class_number]
    return pitchtools.NamedPitch(pitch_number, diatonic_pitch_class_name)



# -*- encoding: utf-8 -*-
from abjad import *


def test_pitchtools_numbered_inversion_equivalent_interval_class_dictionary_01():

    chord = Chord([0, 2, 11], (1, 4))
    vector = pitchtools.numbered_inversion_equivalent_interval_class_dictionary(chord.written_pitches)

    assert vector == {
        0: 0,
        1: 1,
        2: 1,
        3: 1,
        4: 0,
        5: 0,
        6: 0}


def test_pitchtools_numbered_inversion_equivalent_interval_class_dictionary_02():

    staff = Staff("c'8 d'8 e'8 f'8 c'8 d'8 e'8 f'8 c'8 d'8 e'8 f'8")
    pitches = pitchtools.list_named_pitches_in_expr(staff)
    vector = pitchtools.numbered_inversion_equivalent_interval_class_dictionary(pitches)

    assert vector == {
        0: 12,
        1: 9,
        2: 18,
        3: 9,
        4: 9,
        5: 9,
        6: 0}


def test_pitchtools_numbered_inversion_equivalent_interval_class_dictionary_03():

    chord = Chord([-2, -1.5, 9], (1, 4))
    vector = pitchtools.numbered_inversion_equivalent_interval_class_dictionary(chord.written_pitches)

    assert vector == {
        0:   0,
        0.5: 1,
        1:   1,
        1.5: 1,
        2:   0,
        2.5: 0,
        3:   0,
        3.5: 0,
        4:   0,
        4.5: 0,
        5:   0,
        5.5: 0,
        6:   0}



# -*- encoding: utf-8 -*-
from abjad.tools.rhythmmakertools.IncisedRhythmMaker import IncisedRhythmMaker


class OutputIncisedRhythmMaker(IncisedRhythmMaker):
    '''Abstract base class for rhythm-makers that incise only the
    first and last output cells they produce.
    '''

    ### PRIVATE METHODS ###

    def _make_numeric_map(
        self,
        duration_pairs,
        prefix_talea,
        prefix_lengths,
        suffix_talea,
        suffix_lengths,
        prolation_addenda,
        ):
        numeric_map, prefix_talea_index, suffix_talea_index = [], 0, 0
        prefix_length, suffix_length = prefix_lengths[0], suffix_lengths[0]
        prefix = prefix_talea[
            prefix_talea_index:prefix_talea_index+prefix_length]
        suffix = suffix_talea[
            suffix_talea_index:suffix_talea_index+suffix_length]
        if len(duration_pairs) == 1:
            prolation_addendum = prolation_addenda[0]
            numerator = duration_pairs[0][0]
            numerator += (prolation_addendum % numerator)
            numeric_map_part = self._make_numeric_map_part(
                numerator, prefix, suffix)
            numeric_map.append(numeric_map_part)
        else:
            prolation_addendum = prolation_addenda[0]
            if isinstance(duration_pairs[0], tuple):
                numerator = duration_pairs[0][0]
            else:
                numerator = duration_pairs[0].numerator
            numerator += (prolation_addendum % numerator)
            numeric_map_part = self._make_numeric_map_part(
                numerator, prefix, ())
            numeric_map.append(numeric_map_part)
            for i, duration_pair in enumerate(duration_pairs[1:-1]):
                prolation_addendum = prolation_addenda[i+1]
                if isinstance(duration_pair, tuple):
                    numerator = duration_pair[0]
                else:
                    numerator = duration_pair.numerator
                numerator += (prolation_addendum % numerator)
                numeric_map_part = self._make_numeric_map_part(
                    numerator, (), ())
                numeric_map.append(numeric_map_part)
            try:
                prolation_addendum = prolation_addenda[i+2]
            except UnboundLocalError:
                prolation_addendum = prolation_addenda[1+2]
            if isinstance(duration_pairs[-1], tuple):
                numerator = duration_pairs[-1][0]
            else:
                numerator = duration_pairs[-1].numerator
            numerator += (prolation_addendum % numerator)
            numeric_map_part = self._make_numeric_map_part(
                numerator, (), suffix)
            numeric_map.append(numeric_map_part)
        return numeric_map



# -*- encoding: utf-8 -*-


def arg_to_tridirectional_ordinal_constant(arg):
    r'''Convert `arg` to tridirectional ordinal constant:

        >>> stringtools.arg_to_tridirectional_ordinal_constant('^')
        Up

    ::

        >>> stringtools.arg_to_tridirectional_ordinal_constant('_')
        Down

    ::

        >>> stringtools.arg_to_tridirectional_ordinal_constant(1)
        Up

    ::

        >>> stringtools.arg_to_tridirectional_ordinal_constant(-1)
        Down

    If `arg` is Up, Center or Down, `arg` will be returned.

    Returns OrdinalConstant or None.
    '''

    lookup = {
        Up: Up,
        '^': Up,
        'up': Up,
        1: Up,
        Down: Down,
        '_': Down,
        'down': Down,
        -1: Down,
        Center: Center,
        '-': Center,
        0: Center,
        'center': Center,
        'default': Center,
        'neutral': Center,
    }
    if arg is None:
        return None
    elif arg in lookup:
        return lookup[arg]
    raise ValueError



# -*- encoding: utf-8 -*-
from abjad.tools.pitchtools import NamedPitch
from abjad.tools.pitchtools import make_n_middle_c_centered_pitches


def test_pitchtools_make_n_middle_c_centered_pitches_01():
    pitches = make_n_middle_c_centered_pitches(0)
    assert len(pitches) == 0


def test_pitchtools_make_n_middle_c_centered_pitches_02():
    pitches = make_n_middle_c_centered_pitches(2)
    assert pitches == [NamedPitch('b'), NamedPitch("d'")]


def test_pitchtools_make_n_middle_c_centered_pitches_03():
    pitches = make_n_middle_c_centered_pitches(-2)
    assert pitches == [NamedPitch("d'"), NamedPitch('b')]


def test_pitchtools_make_n_middle_c_centered_pitches_04():
    pitches = make_n_middle_c_centered_pitches(3)
    assert pitches == [NamedPitch('a'), NamedPitch("c'"), NamedPitch("e'")]
